"""
<name>KEGG Pathways</name>
<description>Browse KEGG pathways that include an input set of genes.</description>
<priority>2030</priority>
<icon>icons/KEGG.png</icon>
"""

from __future__ import absolute_import, with_statement 

import sys
from collections import defaultdict
import webbrowser

import orange
from Orange.orng import orngMisc, orngServerFiles
from Orange.orng.orngDataCaching import data_hints
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from .. import obiKEGG
from .. import obiTaxonomy
from .. import obiKEGG2 as obiKEGG
from .. import obiGeneSets

USE_THREADING = True

if USE_THREADING:
    import threading
    from functools import partial
    def threading_queued_invoke(qobj, func):
        def safe_wrapper(*args, **kwargs):
            QMetaObject.invokeMethod(qobj, "queuedInvoke", 
                                     Qt.QueuedConnection, 
                                     Q_ARG("PyQt_PyObject",
                                           partial(func, *args, **kwargs)))
        return safe_wrapper
    
def split_and_strip(string, sep=None):
    return [s.strip() for s in string.split(sep)]

class EntryGraphicsItem(QGraphicsPathItem):
    def __init__(self, graphics, *args):
        QGraphicsPathItem.__init__(self, *args)
        path = QPainterPath()
        x, y, w, h = [int(graphics.get(c, 0)) for c in ["x", "y", "width", "height"]]
        type = graphics.get("type", "rectangle")
        if type == "rectangle":
            path.addRect(QRectF(x - w/2, y - h/2, w, h))
        elif type == "roundrectangle":
            path.addRoundedRect(QRectF(x - w/2, y - h/2, w, h), 10, 10)
        elif type == "circle":
            path.addEllipse(QRectF(x - w/2, y - h/2, w, h))
            
        self.setPath(path)
        self.setAcceptHoverEvents(True)
        self._actions = []
        self.link = None
        
    def hoverEnterEvent(self, event):
        self.setBrush(QBrush(QColor(0, 100, 0, 100)))
        
    def hoverLeaveEvent(self, event):
        self.setBrush(QBrush(Qt.NoBrush))
        
    def contextMenuEvent(self, event):
        if self._actions:
            self._menu = menu = QMenu()
            for action in self._actions:
                menu.addAction(action)
            menu.popup(event.screenPos())
            
    def itemChange(self, change, value):
        if change == QGraphicsItem.ItemSelectedHasChanged:
            self.setPen(QPen(Qt.red if self.isSelected() else Qt.blue, 2))
            
        return QGraphicsPathItem.itemChange(self, change, value)
            
class GraphicsPathwayItem(QGraphicsPixmapItem):
    def __init__(self, pathway, objects, *args, **kwargs):
        QGraphicsPixmapItem.__init__(self, *args)
        self.setTransformationMode(Qt.SmoothTransformation)
        self.setPathway(pathway)
        self.setMarkedObjects(objects, name_mapper=kwargs.get("name_mapper", {}))
        
    def setPathway(self, pathway):
        self.pathway = pathway
        if pathway:
            image_filename = pathway.get_image()
            self._pixmap = QPixmap(image_filename)
        else:
            self._pixmap = QPixmap()
        self.setPixmap(self._pixmap)
        
    def setMarkedObjects(self, objects, name_mapper={}):
        for entry in self.pathway.entries() if self.pathway else []:
            if entry.type == "group":
                continue
            graphics = entry.graphics
            contained_objects = [obj for obj in objects if obj in entry.name]
            item = EntryGraphicsItem(graphics, self, self.scene())
            item.setToolTip(self.tooltip(entry, contained_objects, name_mapper))
            item._actions = self.actions(entry, contained_objects)
            item.marked_objects = contained_objects
            if contained_objects:
                item.setPen(QPen(Qt.blue, 2))
                item.setFlag(QGraphicsItem.ItemIsSelectable, True)
    
    def actions(self, entry, marked_objects=[]):
        actions = []
        type = entry.type
        if marked_objects:
            action = QAction("View genes on kegg website", None)
            org = set([s.split(":")[0] for s in marked_objects]).pop()
            genes = [s.split(":")[-1] for s in marked_objects]
            address = "http://www.genome.jp/dbget-bin/www_bget?" + "+".join([org] + genes)
            QObject.connect(action, SIGNAL("triggered()"), lambda toggled=False, address=address: webbrowser.open(address))
            actions.append(action)
        elif hasattr(entry, "link"):
            action = QAction("View %s on KEGG website" % str(type), None)
            QObject.connect(action, SIGNAL("triggered()"), lambda toggled=False, address=entry.link: webbrowser.open(address))
            actions.append(action)
        return actions
    
    def tooltip(self, entry, objects, name_mapper={}):
        names = [obj for obj in objects if obj in entry.name]
        names = [name_mapper.get(name, name) for name in names]
        text = "<p>%s</p>" % (entry.name[:16] + " ..." if len(entry.name) > 20 else entry.name)
        if names:
            text += "<br>".join(names)
        return text
    
    def contextMenuEvent(self, event):
        self._menu = menu = QMenu()
        action = menu.addAction("View this pathway on KEGG website")
        address ="http://www.kegg.jp/kegg-bin/show_pathway?%s%s" % (self.pathway.org, self.pathway.number)
        QObject.connect(action, SIGNAL("triggered()"), lambda : webbrowser.open(address))
        menu.popup(event.screenPos())

class PathwayView(QGraphicsView):
    def __init__(self, master, *args):
        QGraphicsView.__init__(self, *args)
        self.master = master
        
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        
        self.setRenderHints(QPainter.Antialiasing)
        scene = QGraphicsScene(self)
        self.pixmapGraphicsItem = QGraphicsPixmapItem(None, scene) 
        self.setScene(scene)
        
        self.setMouseTracking(True)
        self.viewport().setMouseTracking(True)
        
        self.setFocusPolicy(Qt.WheelFocus)
        
    def SetPathway(self, pathway=None, objects=[]):
        self.scene().clear()
        self.pathway = pathway
        self.objects = objects
        self.pathwayItem = GraphicsPathwayItem(pathway, objects, None, self.scene(), name_mapper=getattr(self.master, "uniqueGenesDict", {}))
        self.scene().setSceneRect(self.pathwayItem.boundingRect())
        self.updateTransform()
    
    def resizeEvent(self, event):
        self.updateTransform()
        return QGraphicsView.resizeEvent(self, event)
            
    def updateTransform(self):
        if self.master.autoResize:
            self.fitInView(self.scene().sceneRect().adjusted(-1, -1, 1, 1), Qt.KeepAspectRatio)
        else:
            self.setTransform(QTransform())
            
    def paintEvent(self, event):
        QGraphicsView.paintEvent(self, event)
        if getattr(self, "_userMessage", None):
            painter = QPainter(self.viewport())
            font = QFont(self.font())
            font.setPointSize(15)
            painter.setFont(font)
            painter.drawText(self.viewport().geometry(), Qt.AlignCenter, self._userMessage)
            painter.end()
        
    
class OWKEGGPathwayBrowser(OWWidget):
    settingsList = ["organismIndex", "geneAttrIndex", "autoCommit", "autoResize", "useReference", "useAttrNames", "caseSensitive", "showOrthology"]
    contextHandlers = {"":DomainContextHandler("",[ContextField("organismIndex", DomainContextHandler.Required + DomainContextHandler.IncludeMetaAttributes),
                                                   ContextField("geneAttrIndex", DomainContextHandler.Required + DomainContextHandler.IncludeMetaAttributes),
                                                   ContextField("useAttrNames", DomainContextHandler.Required + DomainContextHandler.IncludeMetaAttributes)])}
    def __init__(self, parent=None, signalManager=None, name="KEGG Pathways"):
        OWWidget.__init__(self, parent, signalManager, name, wantGraph=True)
        self.inputs = [("Examples", ExampleTable, self.SetData), ("Reference", ExampleTable, self.SetRefData)]
        self.outputs = [("Selected Examples", ExampleTable), ("Unselected Examples", ExampleTable)]
        self.organismIndex = 0
        self.geneAttrIndex = 0
        self.autoCommit = False
        self.autoResize = True
        self.useReference = False
        self.useAttrNames = 0
        self.caseSensitive = True
        self.showOrthology = True
        self.autoFindBestOrg = False
        self.loadSettings()

        self.controlArea.setMaximumWidth(250)
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.infoLabel = OWGUI.widgetLabel(box, "No data on input\n")
        
        self.allOrganismCodes = {} 
        
        self.organismCodes = []
        
        box = OWGUI.widgetBox(self.controlArea, "Organism")
        self.organismComboBox = cb = OWGUI.comboBox(box, self,
                                "organismIndex", 
                                items=[],
                                callback=self.OrganismSelectionCallback, 
                                addSpace=True,
                                debuggingEnabled=0)
        
        self.signalManager.freeze(self).push()
        
        box = OWGUI.widgetBox(self.controlArea, "Gene attribute")
        self.geneAttrCombo = OWGUI.comboBox(box, self, "geneAttrIndex",
                                            callback=self.Update)
        
        OWGUI.checkBox(box, self, "useAttrNames",
                       "Use variable names", 
                       disables=[(-1, self.geneAttrCombo)],
                       callback=self.UseAttrNamesCallback)
        
        self.geneAttrCombo.setDisabled(bool(self.useAttrNames))
        
        OWGUI.separator(self.controlArea)
        
        OWGUI.checkBox(self.controlArea, self, "useReference",
                       "From signal",
                       box="Reference",
                       callback=self.Update)
        
        OWGUI.separator(self.controlArea)

        OWGUI.checkBox(self.controlArea, self, "showOrthology",
                       "Show pathways in full orthology",
                       box="Orthology",
                       callback=self.UpdateListView)
        
        OWGUI.checkBox(self.controlArea, self, "autoResize",
                       "Resize to fit",
                       box="Image",
                       callback=self.UpdatePathwayViewTransform)
        
        box = OWGUI.widgetBox(self.controlArea, "Cache Control")
        
        OWGUI.button(box, self, "Clear cache", 
                     callback=self.ClearCache,
                     tooltip="Clear all locally cached KEGG data.")
        
        OWGUI.separator(self.controlArea)

        box = OWGUI.widgetBox(self.controlArea, "Selection")
        OWGUI.checkBox(box, self, "autoCommit", "Commit on update")
        OWGUI.button(box, self, "Commit", callback=self.Commit, default=True)
        OWGUI.rubber(self.controlArea)
        
        spliter = QSplitter(Qt.Vertical, self.mainArea)
        self.pathwayView = PathwayView(self, spliter)
        self.mainArea.layout().addWidget(spliter)

        self.listView = QTreeWidget(spliter)
        spliter.addWidget(self.listView)
        
        self.listView.setAllColumnsShowFocus(1)
        self.listView.setColumnCount(4)
        self.listView.setHeaderLabels(["Pathway", "P value", 
                                       "Genes", "Reference"])

        self.listView.setSelectionMode(QAbstractItemView.SingleSelection)
            
        self.listView.setSortingEnabled(True)
        
        self.listView.setMaximumHeight(200)
        
        self.connect(self.listView,
                     SIGNAL("itemSelectionChanged()"),
                     self.UpdatePathwayView)
        
        self.connect(self.graphButton,
                     SIGNAL("clicked()"),
                     self.saveGraph)
        
        self.ctrlPressed = False
        self.selectedObjects = defaultdict(list)
        self.data = None
        self.refData = None
        
        self.resize(800, 600)
        
        self.connect(self,
                     SIGNAL("widgetStateChanged(QString, int, QString)"),
                     self.onStateChange)
        
        self.has_new_data = False
        self.has_new_reference_set = False
        
        self.setEnabled(False)
        QTimer.singleShot(100, self.UpdateOrganismComboBox)
        
        
    def UpdateOrganismComboBox(self):
        # First try to import suds
        try:
            import suds
        except ImportError:
            QMessageBox.warning(self,
                "'suds' library required.",
                '<p>Please install \
<a href="http://pypi.python.org/pypi/suds">suds</a> library \
to use KEGG Pathways widget.</p>'
                )
            
        try:
            genome = obiKEGG.KEGGGenome()
            all_codes = list(genome)
            
            self.allOrganismCodes = genome 
    
            essential = genome.essential_organisms()
            common = genome.common_organisms()
            common = [c for c in common if c not in essential]
            
            self.infoLabel.setText("Fetching organism definitions\n")
            
            pb = OWGUI.ProgressBar(self, len(essential + common))
            codes = []
            for i, code in enumerate(essential + common):
                codes.append((code, genome[code].definition))
                pb.advance()
            pb.finish()
            self.organismCodes = codes
            
            items = [desc for code, desc in self.organismCodes]
            
            self.organismCodes = [code for code, desc in self.organismCodes]

            # TODO: Add option to specify additional organisms not 
            # in the common list.

            self.organismComboBox.addItems(items)
        finally:
            self.setEnabled(True)
            self.infoLabel.setText("No data on input\n")
            self.signalManager.freeze(self).pop() #setFreeze(0)

    def Clear(self):
        self.infoLabel.setText("No data on input\n")
        self.listView.clear()
        self.ClearPathway()
        
        self.send("Selected Examples", None)
        self.send("Unselected Examples", None)
        
    def ClearPathway(self):
        self.pathwayView.SetPathway(None)
        self.selectedObjects = defaultdict(list)
        
    def SetData(self, data=None):
        self.closeContext()
        self.data = data
        self.warning(0)
        if data is not None:
            self.SetGeneAttrCombo()
            taxid = data_hints.get_hint(data, "taxid", None)
            if taxid:
                try:
                    code = obiKEGG.from_taxid(taxid)
                    self.organismIndex = self.organismCodes.index(code)
                except Exception, ex:
#                    self.log.exception(self, level=self.log.INFO)
                    print ex, taxid
            
            self.useAttrNames = data_hints.get_hint(data, "genesinrows", self.useAttrNames)
            
            self.openContext("", data)
        else:
            self.Clear()
            
        self.has_new_data = True

    def SetRefData(self, data=None):
        self.refData = data
        self.has_new_reference_set = True
        
    def handleNewSignals(self):
        if self.has_new_data or (self.has_new_reference_set and \
                                 self.useReference):
            self.Update()
            
            self.has_new_data = False
            self.has_new_reference_set = False
        
    def UseAttrNamesCallback(self):
        self.Update()

    def OrganismSelectionCallback(self):
        self.Update()
        
    def SetGeneAttrCombo(self):
        self.geneAttrCandidates = self.data.domain.attributes + \
                                  self.data.domain.getmetas().values()
        self.geneAttrCandidates = filter(lambda v:v.varType in [orange.VarTypes.Discrete,
                                                                orange.VarTypes.String],
                                         self.geneAttrCandidates)
        self.geneAttrCombo.clear()
        
        self.geneAttrCombo.addItems([var.name for var in self.geneAttrCandidates])
            
    def UpdateListView(self):
        self.bestPValueItem = None
        self.listView.clear()
        if not self.data:
            return
        allPathways = self.org.pathways()
        allRefPathways = obiKEGG.pathways("map")
#        self.progressBarFinished()
        items = []
        self.progressBarInit()
        kegg_pathways = obiKEGG.KEGGPathways()
        kegg_pathways.pre_cache(self.pathways.keys(),
                                progress_callback=self.progressBarSet)
        self.progressBarFinished()
        if self.showOrthology:
            self.koOrthology = obiKEGG.KEGGBrite("ko00001")
            self.listView.setRootIsDecorated(True)
            path_ids = set([s[-5:] for s in self.pathways.keys()])
            def _walkCollect(koEntry):
                num = koEntry.title[:5] if koEntry.title else None
                if num  in path_ids:
                    return [koEntry] + reduce(lambda li,c:li+_walkCollect(c), [child for child in koEntry.entries], [])
                else:
                    c = reduce(lambda li,c:li+_walkCollect(c), [child for child in koEntry.entries], [])
                    return c + (c and [koEntry] or [])
            allClasses = reduce(lambda li1, li2: li1+li2, [_walkCollect(c) for c in self.koOrthology], [])
            def _walkCreate(koEntry, lvItem):
                item = QTreeWidgetItem(lvItem)
                id = "path:"+self.organismCodes[min(self.organismIndex, len(self.organismCodes)-1)] + koEntry.title[:5]
                p = kegg_pathways.get_entry(id)
                if koEntry.title[:5] in path_ids:
                    if p is None:
                        # In case the genesets still have obsolete entries 
                        name = koEntry.title
                    else:
                        name = p.name
                    genes, p_value, ref = self.pathways[id]
                    item.setText(0, name)
                    item.setText(1, "%.5f" % p_value)
                    item.setText(2, "%i of %i" %(len(genes), len(self.genes)))
                    item.setText(3, "%i of %i" %(ref, len(self.referenceGenes)))
                    item.pathway_id = id if p is not None else None
                else:
                    item.setText(0, p.name if id in allPathways else koEntry.title)
                    if id in allPathways:
                        item.pathway_id = id
                    elif "path:map" + koEntry.title[:5] in allRefPathways:
                        item.pathway_id = "path:map" + koEntry.title[:5]
                    else:
                        item.pathway_id = None
                
                for child in koEntry.entries:
                    if child in allClasses:
                        _walkCreate(child, item)
            
            for koEntry in self.koOrthology:
                if koEntry in allClasses:
                    _walkCreate(koEntry, self.listView)
                    
            self.listView.update()
        else:
            self.listView.setRootIsDecorated(False)
            pathways = self.pathways.items()
            pathways.sort(lambda a,b:cmp(a[1][1], b[1][1]))
            for id, (genes, p_value, ref) in pathways:
                item = QTreeWidgetItem(self.listView)
                item.setText(0, kegg_pathways.get_entry(id).name)
                item.setText(1, "%.5f" % p_value)
                item.setText(2, "%i of %i" %(len(genes), len(self.genes)))
                item.setText(3, "%i of %i" %(ref, len(self.referenceGenes)))
                item.pathway_id = id
                items.append(item)
                
        self.bestPValueItem = items and items[0] or None
        self.listView.expandAll()
        for i in range(4):
            self.listView.resizeColumnToContents(i)
            
        if self.bestPValueItem:
            self.listView.selectionModel().select(self.listView.indexFromItem(self.bestPValueItem), QItemSelectionModel.ClearAndSelect)

    def UpdatePathwayView(self):
        items = self.listView.selectedItems()
        
        if len(items) > 0:
            item = items[0]
        else:
            item = None
            
        self.selectedObjects = defaultdict(list)
        self.Commit()
        item = item or self.bestPValueItem
        if not item or not item.pathway_id:
            self.pathwayView.SetPathway(None)
            return
        
        if USE_THREADING:
            result = {}
            def call(pathway_id):
                result["pathway"] = p = obiKEGG.KEGGPathway(pathway_id)
                p._get_kgml() # makes sure the kgml file is downloaded
                p._get_image_filename() # makes sure the image is downloaded
                
            self.setEnabled(False)
            try:
                thread = threading.Thread(None,
                                          call,
                                          name="get_kgml_and_image",
                                          args=(item.pathway_id,))
                thread.start()
                while thread.is_alive():
                    thread.join(timeout=0.025)
                    qApp.processEvents()
            finally:
                self.setEnabled(True)
            if "pathway" in result:
                self.pathway = result["pathway"]
            else:
                raise Exception("Could not get kgml and  pathway image")
        else:
            self.pathway = obiKEGG.KEGGPathway(item.pathway_id)
            
        self.pathwayView.SetPathway(self.pathway, self.pathways.get(item.pathway_id, [[]])[0])
            
    def UpdatePathwayViewTransform(self):
        self.pathwayView.updateTransform()
        
    def Update(self):
        if not self.data:
            return
        self.error(0)
        self.information(0)
        pb = OWGUI.ProgressBar(self, 100)
        if self.useAttrNames:
            genes = [str(v.name).strip() for v in self.data.domain.attributes]
        elif self.geneAttrCandidates:
            geneAttr = self.geneAttrCandidates[min(self.geneAttrIndex, len(self.geneAttrCandidates)-1)]
            genes = [str(e[geneAttr]) for e in self.data if not e[geneAttr].isSpecial()]
            if any("," in gene for gene in genes):
                genes = reduce(list.__add__, (split_and_strip(gene, ",") for gene in genes), [])
                self.information(0, "Separators detected in input gene names. Assuming multiple genes per instance.")
        else:
            self.error(0, "Cannot extract gene names from input")
            genes = []
        org_code = self.organismCodes[min(self.organismIndex, len(self.organismCodes)-1)]
        
        if USE_THREADING:
            result = {}
            def callable(*args, **kwargs):
                result["org"] = org = obiKEGG.KEGGOrganism(org_code)
                # Make sure genes are cached for global reference set
                result["genes"] = org.genes.keys()
                
            self.setEnabled(False)
            try:
                thread = threading.Thread(None, callable,
                                          name="get_organism_genes",
                                          )
                thread.start()
                while thread.is_alive():
                    thread.join(timeout=0.025)
                    qApp.processEvents()
            finally:
                self.setEnabled(True)
            
            if "org" in result:
                org = result["org"]
            else:
                raise Exception("Could not get organism genes")
        else:
            org = obiKEGG.KEGGOrganism(org_code)
        
        uniqueGenes, conflicting, unknown = org.get_unique_gene_ids(set(genes), self.caseSensitive)
        genesCount = len(set(genes))
        self.infoLabel.setText("%i unique gene names on input\n%i (%.1f%%) genes names matched" % (genesCount, len(uniqueGenes), 100.0*len(uniqueGenes)/genesCount if genes else 0.0))  
        
        self.information(1)
        if self.useReference and self.refData:
            if self.useAttrNames:
                reference = [str(v.name).strip() for v in self.refData]
            else:
                geneAttr = self.geneAttrCandidates[min(self.geneAttrIndex, len(self.geneAttrCandidates)-1)]
                reference = [str(e[geneAttr]) for e in self.refData if not e[geneAttr].isSpecial()]
                if any("," in gene for gene in reference):
                    reference = reduce(list.__add__, (split_and_strip(gene, ",") for gene in reference), [])
                    self.information(1, "Separators detected in reference gene names. Assuming multiple genes per example.")
            uniqueRefGenes, conflicting, unknown = org.get_unique_gene_ids(set(reference), self.caseSensitive)
            self.referenceGenes = reference = uniqueRefGenes.keys()
        else:
            self.referenceGenes = reference = org.get_genes()
        self.uniqueGenesDict = uniqueGenes
        self.genes = uniqueGenes.keys()
        self.revUniqueGenesDict = dict([(val, key) for key, val in self.uniqueGenesDict.items()])
        
        taxid = obiKEGG.to_taxid(org.org_code)
        r_tax_map = dict((v,k) for k, v in obiKEGG.KEGGGenome.TAXID_MAP.items())
        if taxid in r_tax_map:
            taxid = r_tax_map[taxid]
            
        with orngServerFiles.DownloadProgress.setredirect(self.progressBarSet):
            orngServerFiles.update(obiGeneSets.sfdomain, "index.pck")
            kegg_gs_collections = list(obiGeneSets.collections((("KEGG", "pathways"), taxid)))
        
        if USE_THREADING:
            result = {}
            def callable(*args, **kwargs):
#                result["result"] = org.get_enriched_pathways(*args, **kwargs)
                result["result"] = pathway_enrichment(*args, **kwargs)
            
            self.setEnabled(False)
            try:
                thread = threading.Thread(None, callable,
                                          name="get_enriched_pathways",
                                          args=(kegg_gs_collections,
                                                self.genes, 
                                                reference),
                                          kwargs={"callback": 
                                                  threading_queued_invoke(self,
                                                  lambda value:self.progressBarSet(value))}
                                          )
                
                thread.start()
                while thread.is_alive():
                    thread.join(timeout=0.025)
                    qApp.processEvents()
            finally:
                self.setEnabled(True)
                
            if "result" in result:
                self.pathways = result["result"]
            else:
                raise Exception('Could not get enriched pathways')
            
            
        else:
            self.pathways = org.get_enriched_pathways(self.genes, reference,
                                                      callback=lambda value: self.progressBarSet(value))
        
        self.org = org
        if not self.pathways:
            self.warning(0, "No enriched pathways found.")
        else:
            self.warning(0)
        
        self.UpdateListView()
        pb.finish()
    

    def SelectObjects(self, objs):
        if (not self.selectedObjects or self.ctrlPressed) and not objs:
            return
        if self.ctrlPressed:
            for id, graphics in objs:
                graphics = tuple(sorted(graphics.items()))
                if id in self.selectedObjects[graphics]:
                    self.selectedObjects[graphics].pop(self.selectedObjects[graphics].index(id))
                    if not self.selectedObjects[graphics]:
                        del self.selectedObjects[graphics]
                else:
                    self.selectedObjects[graphics].append(id)
        else:
            self.selectedObjects.clear()
            for id, graphics in objs:
                graphics = tuple(sorted(graphics.items()))
                self.selectedObjects[graphics].append(id)
        if self.autoCommit:
            self.Commit()
            

    def Commit(self):
        if self.data:
            selectedItems = self.pathwayView.scene().selectedItems()
            selectedGenes = reduce(set.union, [item.marked_objects for item in selectedItems], set())
            
            if self.useAttrNames:
                selectedVars = [self.data.domain[self.uniqueGenesDict[gene]] for gene in selectedGenes]
                newDomain = orange.Domain(selectedVars ,0)
                data = orange.ExampleTable(newDomain, self.data)
                self.send("Selected Examples", data)
            elif self.geneAttrCandidates:
                geneAttr = self.geneAttrCandidates[min(self.geneAttrIndex, len(self.geneAttrCandidates)-1)]
                selectedExamples = []
                otherExamples = []
                for ex in self.data:
                    names = [self.revUniqueGenesDict.get(name, None) for name in split_and_strip(str(ex[geneAttr]), ",")]
                    if any(name and name in selectedGenes for name in names):
                        selectedExamples.append(ex)
                    else:
                        otherExamples.append(ex)
                        
                if selectedExamples:
                    selectedExamples = orange.ExampleTable(selectedExamples)
                else:
                    selectedExamples = None
                    
                if otherExamples:
                    otherExamples = orange.ExampleTable(otherExamples)
                else:
                    otherExamples = None
                    
                self.send("Selected Examples", selectedExamples)
                self.send("Unselected Examples", otherExamples)
        else:
            self.send("Selected Examples", None)
            self.send("Unselected Examples", None)
        
    def ClearCache(self):
        from ..obiKEGG2 import caching
        try:
            caching.clear_cache()
        except Exception, ex:
            QMessageBox.warning(self, "Cache clear", 
                ex.args[0])
            
        
    def keyPressEvent(self, key):
        if key.key()==Qt.Key_Control:
            self.ctrlPressed=True
        else:
            OWWidget.keyPressEvent(self, key)

    def keyReleaseEvent(self, key):
        if key.key()==Qt.Key_Control:
            self.ctrlPressed=False
        else:
            OWWidget.keyReleaseEvent(self, key)
            
    def onStateChange(self, stateType, id, text):
        if stateType == "Warning":
            self.pathwayView._userMessage = text
            self.pathwayView.viewport().update()
            
    def saveGraph(self):
        from Orange.OrangeWidgets.OWDlgs import OWChooseImageSizeDlg
        sizeDlg = OWChooseImageSizeDlg(self.pathwayView.scene(), parent=self)
        sizeDlg.exec_()

    @pyqtSignature("queuedInvoke(PyQt_PyObject)")
    def queuedInvoke(self, func):
        func()
        
    def progressBarSet(self, value):
#        print "Enter"
        if not getattr(self, "_in_progress_update", False):
            self._in_progress_update = True
            try:
                OWWidget.progressBarSet(self, value)
            finally:
                self._in_progress_update = False
#        else:
#            print "====="
        
#        print "Exit"
    
    def onDeleteWidget(self):
        """ Called before the widget is removed from the canvas.
        """
        self.org = None
        import gc
        gc.collect() # Force collection
        
    def UpdateToLatestPathways(self):
        pass        
          
from .. import obiProb

def pathway_enrichment(genesets, genes, reference, prob=None, callback=None):
    result_sets = []
    p_values = []
    if prob is None:
        prob = obiProb.Hypergeometric()
        
    for i, gs in enumerate(genesets):
        cluster = gs.genes.intersection(genes)
        ref = gs.genes.intersection(reference)
        k = len(cluster)
        N = len(reference)
        m = len(ref)
        n = len(genes)
        if k:
            p_val = prob.p_value(k, N, m, n)
            result_sets.append((gs.id, cluster, ref))
            p_values.append(p_val)
        if callback is not None:
            callback(100.0 * i / len(genesets))
    
    # FDR correction
    p_values = obiProb.FDR(p_values)
    
    return dict([(id, (genes, p_val, len(ref))) \
                 for (id, genes, ref), p_val in zip(result_sets, p_values)]
                )
    
if __name__=="__main__":
    app = QApplication(sys.argv)
    data = orange.ExampleTable("brown-selected.tab")
    w = OWKEGGPathwayBrowser()
    w.UpdateOrganismComboBox()
##    app.setMainWidget(w)
    w.show()
    w.SetData(orange.ExampleTable(data[:]))
    QTimer.singleShot(10, w.handleNewSignals)
    app.exec_()
    w.saveSettings()



from __future__ import absolute_import, with_statement

import os
import shelve
import datetime
import tarfile
import pickle
from UserDict import DictMixin

from Orange.orng import orngServerFiles

class UpdateShelve(DictMixin):
    def __init__(self, path):
        self.__shelve = shelve.open(path)

    def __expand(self, (func, args)):
        return func.im_class, func.__name__, args

    def __collapse(self, (class_, funcname, args)):
        return getattr(class_, funcname), args

    def __dumps(self, arg):
        return pickle.dumps(self.__expand(arg))

    def __loads(self, arg):
        return self.__collapse(pickle.loads(arg))
        
    def __getitem__(self, arg):
        return self.__shelve.__getitem__(self.__dumps(arg))

    def __setitem__(self, arg, val):
        self.__shelve.__setitem__(self.__dumps(arg), val)

    def __delitem__(self, arg):
        self.__shelve.__delitem__(self.__dumps(arg))

    def keys(self):
        return [self.__loads(key) for key in self.__shelve.keys()]

    def __contains__(self, arg):
        return self.__shelve.__contains__(self.__dumps(arg))

    def __iter__(self):
        for key in self.__shelve:
            yield self.__loads(key)

    def iteritems(self):
        for key, val in self.__shelve.iteritems():
            yield self.__loads(key), val

    def sync(self):
        self.__shelve.sync()
            
from threading import Lock
from functools import wraps

def synchronized(lock):
    def syncfunc(func):
        @wraps(func)
        def f(*args, **kwargs):
            with lock:
                return func(*args, **kwargs)
        return f
    return syncfunc

from datetime import datetime

class Update(object):
    def __init__(self, local_database_path, progressCallback=None):
        """Base update object.
        Each obi module that uses updatable resources should subclass this class and at minimum reimplement
        the IsUpdatable and GetDownloadable methods.
        """
        self.local_database_path = local_database_path
        self.progressCallback = progressCallback
        import os
        try:
            os.mkdir(self.local_database_path)
        except:
            pass
        self.shelve = UpdateShelve(os.path.join(self.local_database_path, ".updates.shelve"))

    def IsUpdatable(self, *args):
        """Return True if the local can be updated else return False
        """
        raise NotImplementedError
    
    def GetLocal(self):
        """Return a list [(callable, args), ...] that have been called in the past.
        """
        return self.shelve.keys()
    
    def GetUpdatable(self):
        """Return a list [(callable, args), ...] that can be updated.
        """
        return [item for item in self.GetLocal() if self.IsUpdatable(*item)]
        
    def GetDownloadable(self):
        """Return a list [(callable, args), ...] that can be downloaded. Must no contain any
        (callable, args) that are returnd by GetLocal.
        """
        raise NotImplementedError
    
    def GetLastUpdateTime(self, callable, args):
        """Returns the last update time of callable with args.
        """
        return self.shelve.get((callable, args), datetime(1,1,1))

    def _update(self, callable, args, date=None):
        """Insert the record of update with date. if date is None use the current date.
        """
        self.shelve[callable, args] = date if date else datetime.now()
        self.shelve.sync()

from Orange.orng import orngServerFiles

class PKGUpdate(Update):
    def __init__(self, domain, wrappedUpdater, *args, **kwargs):
        """Wrap the subclass of Update to download data as packages from our own server.
        Arguments:
            - domain : should be a string that identifies a resource e.g. "kegg", "go" ...
            - wrappedUpdater : instance of a subclass of Update that is wrapped
        Example:
        >>> pkg = PKGUpdate("go", obiGO.Update())
        >>> for item in pkg.GetUpdatable():
        ...     pkg.Apply(*item)
        ...
        >>>
        """
        Update.__init__(self, wrappedUpdater.local_database_path, wrappedUpdater.progressCallback)
        self.domain = domain
        self.serverFiles = orngServerFiles.ServerFiles()
        self.wrappedUpdater = wrappedUpdater
        self.wrappedUpdaterClass = wrappedUpdater.__class__

    def _GetServerFileName(self, func, args):
        return func.__name__+ ("_" + str(args) if args else "") + ".tar.gz"

    def _GetServerDateTime(self, *args):
        time = self.serverFiles.info(self.domain, self._GetServerFileName(*args))["datetime"].split(".")[0]
        return datetime.strptime(time, "%Y-%m-%d %H:%M:%S")
    
    def IsUpdatable(self, *args):
        try:
            return args in self.shelve and self.GetLastUpdateTime(*args) < self._GetServerDateTime(*args)
        except Exception:
            return False
        
    def GetDownloadable(self):
        files = self.serverFiles.listfiles(self.domain)
        downloadable = []
        for filename in files:
            if "_" in filename:
                funcname, args = filename.split("_",1)
                args = args.split(".")[0]
                args = eval(args) 
            else:
                funcname, args = filename.split(".")[0], ()
            downloadable.append((getattr(self.wrappedUpdaterClass, funcname), args))
        return [item for item in downloadable if item not in self.shelve]

    def UpdateWrapper(self, func):
        @wraps(func)
        def update(*args):
            filename = self._GetServerFileName(func, args)
            time = self._GetServerDateTime(func, args)
            orngServerFiles.download(self.domain, filename)
            filepath = orngServerFiles.localpath(self.domain, filename)
            tar = tarfile.open(filepath)
            tar.extractall(self.local_database_path)
            self._update(func, args, time)
        return update

    def Apply(self, func, args):
        """Use this method to apply a return values of GetUpdatable ... which are unwrapped
        """
        if func.im_class == type(self):
            return func(self, *args)
        else:
            return getattr(self, func.__name__)(*args)
            
    def __getattr__(self, name):
        try:
            return self.UpdateWrapper(getattr(self.wrappedUpdaterClass, name))
        except AttributeError:
            raise AttributeError(name)

def firstUpdateConsole():
    from . import obiKEGG, obiGO, obiGenomicsUpdate

    pkgUpdate = obiGenomicsUpdate.PKGUpdate("go", obiGO.Update())

    print "Updating GO ontology"
    pkgUpdate.UpdateOntology()

    for org, name in [("goa_human", "Homo sapiens"), ("sgd", "Yeast")]:
        print "Updating GO anotations for", name
        pkgUpdate.UpdateAnnotation(org)

    pkgUpdate = obiGenomicsUpdate.PKGUpdate("kegg", obiKEGG.Update())

    print "Updating KEGG taxonomy"
    pkgUpdate.UpdateTaxonomy()

    print "Updating KEGG orthology"
    pkgUpdate.UpdateOrthology()

    print "Updating KEGG reference pathways"
    pkgUpdate.UpdateReference()

    for org, name in [("hsa", "Homo sapiens"), ("sce", "Yeast")]:
        print "Updating KEGG pathways for", name
        pkgUpdate.UpdateOrganism(org)
    

def firstUpdateQt():
    pass
    



from __future__ import absolute_import

import random
from math import log
from operator import itemgetter

import numpy

import Orange

from . import obiGEO
from .obiExpression import ExpressionSignificance_Test

# Utility functiions

log2 = lambda x: log(x, 2.)

def my_ratio(x, y):
    """ compute the log-ratio """
    return log2(x/y)

def sign(x):
    return cmp(x, 0)

def common_domain(data1, data2):
    """Use only attributes that are in both data sets"""
    atts = sorted(set([a.name for a in data1.domain.attributes]).intersection(
               [a.name for a in data2.domain.attributes]))
    new1 = Orange.data.Table(Orange.data.Domain(atts + [data1.domain.classVar], data1.domain), data1)
    new2 = Orange.data.Table(Orange.data.Domain(atts + [data2.domain.classVar], data2.domain), data2)
    return new1, new2

def common_genes(data1, data2, att='gene'):
    common_set = list(set(ex[att].value for ex in data1).intersection(ex[att].value for ex in data2))
    print len(set(ex[att].value for ex in data1))
    print len(common_set)
    return data1.filter(**{att: common_set}), data2.filter(**{att: common_set})

# Normalization

def quantilenorm(data):
    """normalization of microarray data to obtain the same distribution in all chips"""
    chips = data.domain.attributes
    genes = [str(d["gene"]) for d in data]
    d_sort = {}
    for c in chips:
        dc = [(float(d[c]), str(d["gene"])) for d in data]
        dc.sort()
        d_sort[c.name] = dc
    d_sort2 = dict([(str(d["gene"]),{}) for d in data])
    for i in range(len(data)):
        genes_c = [(d_sort[c.name][i][1], c.name) for c in chips]
        mean_row = numpy.mean([d_sort[c.name][i][0] for c in chips])
        for g, d in genes_c:
            d_sort2.get(g, {}).update(dict([(d, mean_row)]))
    data_norm = Orange.data.Table(data.domain)
    for i, d in enumerate(data):
        g = str(d["gene"])
        ex = [d_sort2[g][c.name] for c in chips]
        data_norm.append(ex)
        data_norm[i]["gene"] = g
    return data_norm

def get_mediannorm(modeldata):
    """returns the function for median scale normalization"""
    globalmedian = numpy.median([float(d[c]) for d in modeldata for c in modeldata.domain.attributes])
    def normalize(data):
        chips = data.domain.attributes
        medians = [numpy.median([float(d[c]) for d in data]) for c in chips]
        data_norm = Orange.data.Table(data.domain)
        data_norm.domain.add_metas(data.domain.get_metas())
        for i, d in enumerate(data):
            ex = [(d[c.name].value*globalmedian)/medians[k] for k,c in enumerate(chips)]
            data_norm.append(ex)
            for m in data.domain.get_metas():
                data_norm[i][m] = data[i][m]
        return data_norm
    return normalize

def medianscalenorm(data, newsamples=None, intersection=True):
    """normalization of microarray data to center the distributions in all chips on the same global median"""
    if not newsamples:
        n = get_mediannorm(data)
        return n(data), None
    else:
        if intersection:
            idata, inewsamples = common_genes(data, newsamples)
        n = get_mediannorm(idata)
        return n(idata), n(inewsamples)

def normalize(data1, data2=None, type='median'):
    if type == 'quantile':
        ndata1 = quantilenorm(data1)
        return ndata1, medianscalenorm(ndata1, data2)[1] if data2 else None
    elif type == 'median':
        return medianscalenorm(data1, data2)
    else:
        return Error


# Gene filtering

def compute_diff_pairs(at_a, d):
    """ computes the pairwise differences between the replicates """
    differ = []
    for i in range(len (at_a)-1):
        for j in range(i+1, len(at_a)):
            differ.append(log2(d[at_a[i]])-log2(d[at_a[j]]))
    return differ

def costruct_series(data, attr_set, differences=False):
    """ Constructs the time series by averaging the replicates of the same time point """
    serie = dict([(str(d["gene"]), [numpy.mean([d[at] for at in at_samples])
                    for at_name, at_samples in attr_set]) for d in data])
    if differences:
        """store the differences between replicates while creating the time series"""
        differences = []
        r = [[differences.extend(compute_diff_pairs(at_samples, d))
              for at_name, at_samples in attr_set] for d in data]
        return serie, differences
    else:
        return serie

def costruct_control_series(serie, num_points, control):
    """ Creation of the control series (for each gene) as a constant profile equal to expression at time 0 """
    ##print "control series..."
    serie_0 = {}
    if control == "avg":
        serie_0.update(dict([(g,[numpy.mean(serie[g]) for i in range(num_points)]) for g in serie]))
    elif control == "t0":
        serie_0.update(dict([(g,[serie[g][0] for i in range(num_points)]) for g in serie]))
    return serie_0

def compute_area(vals, t, baseline=0.):
    return sum(a_pair((t[i], vals[i]), (t[i+1], vals[i+1]), baseline) \
        for i in xrange(len(vals)-1))

def a_pair(p1, p2, baseline):
    """Area under the line bounded by a pair of two points (x,y) with
    respect to baseline. Both parts around the diagonal are
    positive."""
    x1,y1 = p1
    x2,y2 = p2
    x2 = x2-x1 #same start
    x1 = 0.0
    a = y1-baseline
    b = y2-baseline
    if a*b >= 0: #both on one side
        return abs(x2*(b+a)/2.0)
    else:
        xp = -a * x2 / float(y2-y1)
        return (abs(xp * a) + abs((x2-xp)*b)) / 2.0

def uniform_time_scale(attr_set):
    """ Obtains time points with a unique measure (the lowest among [min,h,d]) present in data"""
    ord_measures = ["min","h","d"]
    converter = {"min":{"h":60, "d":24*60}, "h": {"d":24}}
    measures = list(set([t.split(" ")[1] for t,s in attr_set]))
    if len(measures) == 1:
        time_points = [float(t.split(" ")[0]) for t,s in attr_set]
    else:
        first_measure = min([(ord_measures.index(m),m) for m in measures])[1]
        time_points = [float(t.split(" ")[0]) for t, s in attr_set
                       if t.split(" ")[1] == first_measure]
        time_points.extend([float(t.split(" ")[0]) * converter[first_measure][t.split(" ")[1]]
                            for t, s in attr_set if t.split(" ")[1] != first_measure])
    return time_points

def AREA(data, attr_set, control='t0', weighted=False, auto=False, perc=99):
    """ AREA (Area Under the Curve) filtering method """
    from matplotlib.mlab import prctile
    if weighted:
        time_points = uniform_time_scale(attr_set)
    else:
        time_points = range(len(attr_set))

    # Monte Carlo approach to create the null distribution of the areas
    if auto: # null distribution
        serie, differences = costruct_series(data, attr_set, auto)
        serie_campionata = {}
        area = []
        for i in range(20000):
            serie_campionata = random.sample(differences, len(time_points))
            area.append(compute_area(serie_campionata, time_points))
        area_threshold = prctile(area, perc)
    else:
        serie = costruct_series(data, attr_set, auto)

    serie_0 = costruct_control_series(serie, len(time_points), control)

    # Gene filtering
    areas = []
    for g in serie:
        diff_s = [log2(serie[g][i]) - log2(serie_0[g][i]) for i in range(len(time_points))]
        area_diff = compute_area(diff_s, time_points);
        if not auto or area_diff > area_threshold:
            areas.append((g, area_diff))
    return areas

class ExpressionSignificance_AREA(ExpressionSignificance_Test):
    def __call__(self, target=None):
        attr_set = {}
        for a in self.data.domain.attributes:
            attr_set[a.attributes['time']] = attr_set.get(a.attributes['time'], []) + [a.name]
        scores = AREA(self.data, sorted(attr_set.items()))
        gene2ind = dict((g, i) for i,g in enumerate(ex['gene'].value for ex in self.data))
        return [(gene2ind[g], s) for g, s in scores]

def FC(data, attr_set, control='t0', thr=2, auto=False, p_thr=0.2):
    """ Gene filtering based on the number of FC of all time points with the control series > thr """
    serie = costruct_series(data, attr_set, False)
    num_points = len(serie.values()[0])
    serie_0 = costruct_control_series(serie, num_points, control)
    fc = [(g, len([0 for i in range(num_points) if abs(my_ratio(s[i], serie_0[g][i])) >= thr]))
          for g, s in serie.items()]
    if auto:
        thr_points = round(p_thr*num_points)
        fc = [(g, v) for g, v in fc if v >= thr_points]
    return fc

class ExpressionSignificance_FCts(ExpressionSignificance_Test):
    def __call__(self, target=None):
        attr_set = {}
        for a in self.data.domain.attributes:
            attr_set[a.attributes['time']] = attr_set.get(a.attributes['time'], []) + [a.name]
        scores = FC(self.data, sorted(attr_set.items()))
        return zip(self.keys, map(itemgetter(1), scores))

def spearmanr_filter(data, limit=1000):
    """ Spearman ranks gene filtering """
    from scipy.stats import spearmanr
    time = [a.attributes['time'] for a in data.domain.attributes]
    exs = sorted(data, reverse=True, key=lambda ex: abs(spearmanr([a.value for a in ex], time)[0]))
    return [str(ex['gene']) for ex in exs[:limit]]


def signed_PCA(data):
    pca = Orange.projection.linear.PCA(data, standardize=False, max_components=1)
    classifier = lambda X: [x[0].value for x in pca(X)]
    predictions = classifier(data)
    classes = [ex.getclass().value for ex in data]
    n = 0
    for i1,c1 in enumerate(classes):
        for i2,c2 in enumerate(classes[:i1]):
            n += cmp(c1,c2) * cmp(predictions[i1], predictions[i2])
    if n < 0:
        def invert(X):
            y = classifier(X)
            return -y if type(y) == float else [-x for x in y]
        return invert
    else:
        return classifier

signed_PCA.name = 'PCA'


def conttime(data, d):
    for a in data.domain.attributes:
        a.attributes['time'] = d[a.attributes['time']]

def conv(attr_set, ticks=True):
    """Obtain time points with a unique measure (the lowest among [min,h,d]) present in data"""
    ord_measures = ["min","h","d"]
    converter = {"min":{"h":60, "d":24*60}, "h": {"d":24}}
    measures = list(set([t.split(" ")[1] for t in attr_set]))
    if len(measures) == 1:
        time_points = [(t, float(t.split(" ")[0])) for t in attr_set]
    else:
        first_measure = min([(ord_measures.index(m),m) for m in measures])[1]
        time_points = [(t, float(t.split(" ")[0])) for t in attr_set if t.split(" ")[1] == first_measure]
        time_points.extend([(t, float(t.split(" ")[0]) * converter[first_measure][t.split(" ")[1]])
                            for t in attr_set if t.split(" ")[1] != first_measure])
    time_points.sort(key=itemgetter(1))
    if ticks:
        time_points = [(t[0],float(i)) for i,t in enumerate(time_points)]
    return dict(time_points)


def get_projections(data1, data2=None):
    labels1 = list(a.attributes['time'] for a in data1.domain.attributes)
    tdata1 = obiGEO.transpose(data1)
    if data2:
        labels2 = list('[%s]' % a.attributes['time'] for a in data2.domain.attributes)
        tdata2 = obiGEO.transpose(data2)
        tdata1, tdata2 = common_domain(tdata1, tdata2)
        classifier = signed_PCA(tdata1)
        proj1 = classifier(tdata1)
        proj2 = classifier(tdata2)
    else:
        classifier = signed_PCA(tdata1)
        proj1 = classifier(tdata1)
        proj2, labels2 = [], []
    return proj1, labels1, proj2, labels2


############

if False and __name__ == '__main__':
    from . import obiGEO
    # Data set 1
    data1 = obiGEO.GDS('GDS2666').getdata(report_genes=True, transpose=False)
    labels1 = list(a.attributes['time'] for a in data1.domain.attributes)
    attr_set = list(set(a.attributes['time'] for a in data1.domain.attributes))
    convd = conv(attr_set)
    conttime(data1, convd)
    # Data set 2
    data2 = obiGEO.GDS('GDS2667').getdata(report_genes=True, transpose=False)
    labels2 = list(a.attributes['time'] for a in data2.domain.attributes)
    attr_set = list(set(a.attributes['time'] for a in data2.domain.attributes))
    convd = conv(attr_set)
    conttime(data2, convd)
    # Normalize data set 1
    ndata1, _ = normalize(data1, type='quantile')
    # Filtering
    attr_set = {}
    for a in ndata1.domain.attributes:
        attr_set[a.attributes['time']] = attr_set.get(a.attributes['time'], []) + [a.name]
    scores = AREA(ndata1, sorted(attr_set.items()))
    genes = map(itemgetter(0), sorted(scores, key=itemgetter(1))[:1000])
    fdata1 = ndata1.filter(gene=genes)
    # Rescale data set 2
    ttrain, ttest = normalize(fdata1, data2)
    # Model construction and prediction
    # train = obiGEO.transpose(ttrain)
    # test = obiGEO.transpose(ttest)
    # cdtrain, cdtest = common_domain(train, test)
    # classifier = signed_PCA(cdtrain)
    # proj1 = classifier(cdtrain)
    # proj2 = classifier(cdtest)






"""
Return types from api classes interface for the SOAP kegg api.

"""

from datetime import datetime
from collections import namedtuple

Definition = namedtuple(
    "Definition", ["entry_id",
                   "definition"
                   ])

def _Definition_from_items(items):
    """ Definition 'items' tuple from a list of items
    """
    items_list = []
    for name, val in items:
        if isinstance(name, list):
            name = name[0]
        if isinstance(val, list):
            val = val[0]
        items_list.append((str(name), str(val)))
    return Definition(**dict(items_list))

Definition.from_items = staticmethod(_Definition_from_items)
    
BInfo = namedtuple(
    'BInfo', ["entry_id",
              "definition",
              "name",
              "release",
              "curator",
              "contents",
              "last_update",
              "supported_formats"
              ])

def _BInfo_from_text(text):
    """ Parse the return string from binfo into a new BInfo instance.
    """
    lines = text.splitlines()
    name, definition = lines[0].split(" ", 1)
    definition = definition.strip()
    
    entry_id, release = lines[1].split(" ", 1)
    _, release = release.strip().split(" ", 1)
    curator = lines[2].strip()
    contents = lines[3].strip()
    _, last_update = lines[4].strip().split(":",1)
    last_update = datetime.strptime(last_update.strip(), "%y/%m/%d").date()
    supported_formats = lines[5].strip()
    return BInfo(entry_id, definition, name, release, curator,
                 contents, last_update, supported_formats)

BInfo.from_text = staticmethod(_BInfo_from_text)


SSDBRelation = namedtuple(
    "SSDBRelation", ["genes_id1",
                     "genes_id2",
                     "sw_score",
                     "bit_score",
                     "identity",
                     "overlap",
                     "start_position1",
                     "end_position1",
                     "start_position2",
                     "end_position2",
                     "best_flag_1to2",
                     "best_flag_2to1",
                     "definition1",
                     "definition2",
                     "length1",
                     "length2"
                     ])

MotifResult = namedtuple(
    "MotifResult", ["motif_id",
                    "definition",
                    "genes_id",
                    "start_position",
                    "end_position",
                    "score",
                    "evalue"
                    ])

LinkDBRelation = namedtuple(
    "LinkDBRelation", ["entry_id1",
                       "entry_id2",
                       "type",
                       "path"
                       ])

PathwayElement = namedtuple(
    "PathwayElement", ["element_id",
                       "type",
                       "names",
                       "components"
                       ])

PathwayElementRelation = namedtuple(
    "PathwayElementRelation", ["element_id1",
                               "element_id2",
                               "type",
                               "subtypes"
                               ])

Subtype = namedtuple(
    "Subtype", ["element_id",
                "relation",
                "type",
                ])




"""<name>GeneMANIA</name>
<icon>icons/GeneMANIA.png</icon>
"""

from __future__ import absolute_import

import os, sys
import multiprocessing
import random

import Orange
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from .. import obiGeneMania

class BarItemDelegate(QStyledItemDelegate):
    BarRole = OWGUI.OrangeUserRole.next() 
    BarForegroundRole = OWGUI.OrangeUserRole.next()
    
    def __init__(self, parent, brush=QBrush(QColor(255, 170, 127)), scale=(0.0, 1.0)):
        QStyledItemDelegate.__init__(self, parent) 
        self.brush = brush
        self.scale = scale
        
        
    def paint(self, painter, option, index):
        painter.save()
        qApp.style().drawPrimitive(QStyle.PE_PanelItemViewRow, option, painter)
        qApp.style().drawPrimitive(QStyle.PE_PanelItemViewItem, option, painter)
        rect = option.rect
        try:
            val, ok = index.data(self.BarRole).toDouble()
            if ok:
                color = index.data(self.BarForegroundRole)
                if color.isValid() and color.type() == QVariant.Color:
                    brush = QBrush(color)
                else:
                    brush = self.brush
                    
                minval, maxval = self.scale
                val = (val - minval) / (maxval - minval)
                painter.save()
                if option.state & QStyle.State_Selected:
                    painter.setOpacity(0.75)
                painter.setBrush(brush)
                painter.drawRect(rect.adjusted(1, 1, max(-rect.width() * (1.0 - val) - 2, - rect.width() + 2), -2))
                painter.restore()
        except Exception, ex:
            print >> sys.stderr, ex
            
        text = index.data(Qt.DisplayRole).toString()
        if text:
            align, ok = index.data(Qt.TextAlignmentRole).toInt()
            if not ok:
                align = Qt.AlignVCenter | Qt.AlignLeft
                
            painter.drawText(option.rect, align, text)
        painter.restore()
        
        
    def sizeHint(self, option, index):
        size = QStyledItemDelegate.sizeHint(self, option, index)
        metrics = QFontMetrics(option.font)
        height = metrics.lineSpacing() + 2
        return QSize(size.width(), height)
    

class OWGeneMania(OWWidget):
    contextHandlers = {"": DomainContextHandler("", ["selectedOrganismIndex", "selectedGeneAttrIndex", "genesInColumns"])}
    settingsList = ["serverAddress", "selectedOrganismIndex", "selectedGeneAttrIndex", "genesInColumns", "selectedMethodIndex", "resultCount"]
    def __init__(self, parent=None, signalManager=None, name="GeneMANIA"):
        OWWidget.__init__(self, parent, signalManager, name, wantMainArea=True)
        
        self.inputs = [("Input Genes", ExampleTable, self.setData)]
        self.outputs = [("Network", Orange.network.Graph, Default), ("Items", ExampleTable)]
        
        self.serverAddress = obiGeneMania.DEFAULT_SERVER
        self.selectedOrganismIndex = 0
        self.selectedGeneAttrIndex = 0
        self.genesInColumns = False
        self.selectedMethodIndex = 1
        self.resultCount = 10
        
        self.data = None
        
        self.loadSettings()
        
        #####
        # GUI
        #####
        
        self.organisms = [("H. sapiens", "9606"),
                          ("A. thaliana", "3702"),
                          ("C. elegans", "6239"),
                          ("D. melanogaster", "7227"),
                          ("M. musculus", "10090"),
                          ("S. cerevisiae", "4932")]
        
        box = OWGUI.widgetBox(self.controlArea, "Info", addSpace=True)
        self.info = OWGUI.widgetLabel(box, "No genes on input.")
        self.infoState = OWGUI.widgetLabel(box, "")
        self.infoState.setWordWrap(True)
        
        box = OWGUI.widgetBox(self.controlArea, "GeneMANIA server address", addSpace=True)
        OWGUI.lineEdit(box, self, "serverAddress")
        
        self.organismCombo = OWGUI.comboBox(self.controlArea, self, "selectedOrganismIndex", "Organims",
                                            items=[t[0] for t in self.organisms],
                                            tooltip="Select the organism",
                                            callback=self.updateInfo)
        
        box = OWGUI.widgetBox(self.controlArea, "Genes", addSpace=True)
        self.geneAttrCombo = OWGUI.comboBox(box, self, "selectedGeneAttrIndex",
                                            tooltip="Select the attribute with gene names",
                                            callback=self.updateInfo)
        
        cb = OWGUI.checkBox(box, self, "genesInColumns", "Use attribute names",
                            tooltip="Use attribute names as gene names instead.",
                            callback=self.updateInfo,
                            disables=[(-1, self.geneAttrCombo)])
        cb.makeConsistent()
        self.methodItems = [("Automatic relevance", "automatic_relevance"),
                            ("Automatic", "automatic"),
                            ("Biological process", "bp"),
                            ("Molecular function", "mf"),
                            ("Cellular component", "cc"),
                            ("Average", "average"),
                            ("Average category", "average_category")]
        
        toolTips = ["Assigned based on query genes",
                    "Automatically selected weighting method",
                    "Biological process based",
                    "Molecular function based",
                    "Cellular component based",
                    "Equal by data type",
                    "Equal by network"]
        
        OWGUI.comboBox(self.controlArea, self, "selectedMethodIndex", 
                       box="Net combining method",
                       items=[t[0] for t in self.methodItems],
                       callback=self.updateInfo
                       )
        
        OWGUI.spin(self.controlArea, self, "resultCount", 1, 100, 
                   box="Number of gene results",
                   callback=self.updateInfo
                   )
        
        self.geneManiaLinkLabel = OWGUI.widgetLabel(self.controlArea, "")
        self.geneManiaLinkLabel.setOpenExternalLinks(True)
        OWGUI.button(self.controlArea, self, "Retrieve", callback=self.retrieve)
        OWGUI.rubber(self.controlArea)
        
        self.networksReport = QTreeView()
        self.networksReport.setEditTriggers(QTreeView.NoEditTriggers)
        box = OWGUI.widgetBox(self.mainArea, "Networks")
        box.layout().addWidget(self.networksReport)
        
        self.resize(100, 200)
        
        self.connect(self, SIGNAL("widgetStateChanged(QString, int, QString)"), self.updateInfo)


    def setData(self, data=None):
        self.error([0,1,2])
        self.warning([0])
        self.data = data
        self.closeContext("")
        self.geneAttrCombo.clear()
        self.candidateGeneAttrs = []
        self.selectedGeneAttrIndex = 0
        
        if data is not None:
            self.candidateGeneAttrs = data.domain.variables + data.domain.getmetas().values()
            self.candidateGeneAttrs = [attr for attr in self.candidateGeneAttrs if attr.varType != orange.VarTypes.Continuous]
            self.geneAttrCombo.addItems([attr.name for attr in self.candidateGeneAttrs])
            self.openContext("", data)
        else:
            self.send("Network", None)
            self.send("Items", None)
            
        self.updateInfo()
            
    def onGeneSourceSelection(self):
        genes = self.getSelectedGenes()
        self.info.setText("")
        
    def updateInfo(self, *args):
        if self.data is not None:
            genes = self.getSelectedGenes()
            htmlState = self.widgetStateToHtml()
            self.info.setText("%i genes on input." % len(genes))
            self.infoState.setText(htmlState)
        else:
            self.info.setText("No data on input.")
            self.infoState.setText("")
        
        if self.data:
            org = self.organisms[self.selectedOrganismIndex][1]
            genes = self.getSelectedGenes()
            method = self.methodItems[self.selectedMethodIndex][1]
            conn = obiGeneMania.Connection(self.serverAddress)
            self.geneManiaLinkLabel.setText('<a href="%s">View network in external browser</a>' % conn._queryPage(org, genes, method, self.resultCount))
        else:
            self.geneManiaLinkLabel.setText('')
        
    def getSelectedGenes(self):
        if self.data is not None:
            if self.genesInColumns:
                names = [attr.name for attr in self.data.domain.attributes]
            else:
                attr = self.candidateGeneAttrs[self.selectedGeneAttrIndex]
                names = set([str(ex[attr]) for ex in self.data if not ex[attr].isSpecial()])
            return names
        else:
            return []
        
        
    def retrieve(self):
        org = self.organisms[self.selectedOrganismIndex][1]
        genes = self.getSelectedGenes()
        method = self.methodItems[self.selectedMethodIndex][1]
        self.error([0, 1, 2])
        self.warning([0])
        
        conn = obiGeneMania.Connection(self.serverAddress)
        errorCode, invalid, genes = conn.validate(org, genes)
        if not genes:
            self.error(2, "No valid gene names!")
            self.net, self.netTab = None, None
            self.updateNetworksReport()
            return
        elif invalid:
            self.warning(0, "There are invalid gene names on input:\n%s" % (",".join(invalid[:5])) + (" ..." if len(invalid) > 5 else ""))
            
#        print conn._queryPage(org, genes, method, self.resultCount)
        call = self.asyncCall(conn.retrieveXML, (org, genes), {"m": method, "r": self.resultCount})
        call()
        
        self.progressBarInit()
        self.setEnabled(False)
        try:
#            net = conn.retrieve(org, genes, m=method, r=self.resultCount)
            xml = call.get_result(processEvents=True)
            # Parse the xml in the main thread (pyexpat frequently crashes in 
            # Qt's threading model)
            dom = obiGeneMania.minidom.parseString(xml)
            net = obiGeneMania.parse(dom)
            items = net.items()
        except Exception, ex:
            self.error(0, "Failed to retrieve network from server!\n" + str(ex))
            sys.excepthook(*sys.exc_info())
            net = None
            items = None
        
        if net:
            try:
                self.netTab = obiGeneMania.parsePage(conn._page)
            except Exception:
                self.error(1, "Failed to parse network tabs!\n" + str(ex))
                self.netTab = None
        else:
            self.netTab = None
        self.setEnabled(True)
        self.progressBarFinished()
        
        self.net = net
        
        self.send("Network", net)
        self.send("Items", items)
        
        self.updateNetworksReport()
        
    def updateNetworksReport(self):
        model = QStandardItemModel(self)
        model.setHorizontalHeaderLabels(["Networks", "Weight"])
        root = model.invisibleRootItem()
        def toFloat(s):
            if s.strip().endswith("%"):
                return float(s.strip()[:-1])
            else:
                return float(s)
            
        if self.netTab and self.net and self.net.links:
            for group in self.netTab:
                groupItem = QStandardItem(group.name)
                groupWeightItem = QStandardItem("%.2f %%" % toFloat(group.weight))
                groupWeightItem.setData(QVariant(toFloat(group.weight) / 100.0), BarItemDelegate.BarRole)
                root.appendRow([groupItem, groupWeightItem])
                for net in group.networks:
                    netItem = QStandardItem(net.name)
                    netItem.setData(QVariant(net.description), Qt.ToolTipRole)
                    
                    netWeightItem = QStandardItem("%.2f %%" % toFloat(net.weight))
                    netWeightItem.setData(QVariant(toFloat(net.weight) / 100.0), BarItemDelegate.BarRole)
                    netWeightItem.setData(QVariant(net.description), Qt.ToolTipRole)
                    groupItem.appendRow([netItem, netWeightItem])
                
        self.networksReport.setModel(model)
        self.networksReport.setItemDelegateForColumn(1, BarItemDelegate(self))
        self.networksReport.resizeColumnToContents(0)
            
        
if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = OWGeneMania()
    w.show()
    data = orange.ExampleTable("../../../doc/datasets/brown-selected.tab")
    w.setData(orange.ExampleTable(data[:3]))
    app.exec_()
    w.saveSettings()
        
        



"""
Module for browsing any analyzing sets annotated with MeSH ontology.
"""

from xml.sax import make_parser
from xml.dom import minidom
from xml.sax.handler import ContentHandler
from math import log,exp
from urllib import urlopen
from sgmllib import SGMLParser
import os.path

import orange
from Orange.orng import orngServerFiles

FUZZYMETAID = -12

class obiMeSH(object):
	def __init__(self):
		"""
		Function will initialize obiMeSH module and (in case of no MeSH ontology data) download necessary data using orngServerFiles.
		"""
		#print "SVN version"
		# we check if all files from MeSH directory are localy present. if not, download them
		d = orngServerFiles.ServerFiles()
		f = d.listfiles('MeSH')
		l = orngServerFiles.listfiles('MeSH')
		for i in f:
			if not i in l:
				orngServerFiles.download('MeSH',i)
				print 'Downloading MeSH ontology and chemical annotation. Please wait.', i
		
		self.reference = None
		self.cluster = None
		self.ratio = 1
		self.statistics = None
		self.calculated = False
		
		self.ref_att = "Unknown"
		self.clu_att = "Unknown"
		self.solo_att = "Unknown"
		
		#we calculate log(i!)
		self.lookup = [0]
		for i in range(1, 8000):
			self.lookup.append(self.lookup[-1] + log(i))
		self.dataLoaded = self.__loadOntologyFromDisk()

	def expandToFuzzyExamples(self, examples, att, a, b):
		"""
		Function will return new 'fuzzy' example table. Every example from the input table will get two additional meta attributes ('fuzzy set' and 'u') \
		based on 'a' and 'b' threshold (lower and higher) and attribute 'att'. Attribute 'fuzzy set' indicates name of the fuzzy set while atribute 'u' \ 
		reflects example's degree of membership to particular fuzzy set. Note that input examples with values of 'att' lying on the (a,b) will be expanded \
		into two fuzzy examples.
		"""
		mu = orange.FloatVariable("u")
		mv = orange.StringVariable("fuzzy set")
		examples.domain.addmeta(FUZZYMETAID, mu)
		examples.domain.addmeta(FUZZYMETAID-1, mv)
		newexamples = []
		for j in range(0,len(examples)):
			i = examples[j]
			v = float(i[att])
			if v > a and v < b:  # we have to expand this example
				newexamples.append(i)
				i["fuzzy set"] = 'yes'
				i["u"] = (v-a)/(b-a)
				examples.append(i)
				examples[-1]["fuzzy set"] = "no"
				examples[-1]["u"] = (b-v)/(b-a)
			else:
				if v > a:  #		u(yes) = 1.0
					i["fuzzy set"] = 'yes'
					i["u"] = 1.0
				else: #		u(no) = 1.0
					i["fuzzy set"] = 'no'
					i["u"] = 1.0
		return examples

	def findSubset(self, examples, meshTerms, callback=None, MeSHtype='term'):
		"""
		Function findSubset will return new example table containing examples which are annotated with at least one MeSH term from list 'meshTerms'.
		"""
		# clone
		newdata = orange.ExampleTable(examples.domain)
		self.solo_att = self.__findMeshAttribute(examples)
		ids = list()
		l = len(examples)
		c = 0.0
		meshTerms = list(set(meshTerms))
		# we couldn't find any mesh attribute
		if self.solo_att == "Unknown":
			return newdata
		if MeSHtype == 'term':
			for i in meshTerms:
				ids.extend(self.toID[i])
		else:
			ids = meshTerms
		for e in examples:
			try:
				if callback and c%10 == 0:
					callback(int(c*100.0/l))
				c = c + 1.0
				ends = eval(e[self.solo_att].value)
			except SyntaxError:
				#print "Error in parsing ", e[self.solo_att].value
				continue
			endids = list()
			for i in ends:
				if self.toID.has_key(i):
					endids.extend(self.toID[i])
			#allnodes = self.__findParents(endids)
			allnodes = endids
			# calculate intersection
			isOk = False
			for i in allnodes:
				if ids.count(i) > 0:
					isOk = True
					break

			if isOk:	  # intersection between example mesh terms and observed term group is None
				newdata.append(e)
		return newdata

	def findTerms(self, ids, idType="cid", callback=None):
		"""
		Function findTerms returns a dictionary containing annotated items from the list 'ids'.
		"""
		ret = dict()
		if(not self.dataLoaded):
			print "Annotation and ontology has never been loaded!"
			return ret

		if idType=="cid": 
			for i in ids:
				if self.fromCID.has_key(int(i)): # maybe it is on our database
					ret[int(i)] = self.fromCID[int(i)]
				else:   # no it is not, let's go on the internet - use pubChemAPI
					r = pubChemAPI()
					l = r.getMeSHterms(i)
					ca = r.getPharmActionList(i)
					l.extend(ca) # we extend it with Pharmacological actions
					#l = self.findTermsForCID(i)
					self.fromCID[int(i)] = l
					ret[int(i)] = l
					
					#if len(l)>0:
					#	# if we found something lets save it to a file
					#   TODO: Never use __file__ to access data, but pkg_resources so that it works also in eggs
					#	__dataPath = os.path.join(os.path.dirname(__file__), self.path)
					#	fileHandle = open(os.path.join(__dataPath,'cid-annotation.dat'), 'a')
					#	for s in l:
					#		fileHandle.write('\n' + str(i) + ';' + s )
					#	fileHandle.close()
			return ret
		elif idType == "pmid":  #FIXME PMID annotation
			database = self.fromPMID
		else:
			return ret
		
		for i in ids:
			if(database.has_key(int(i))):
				ret[i] = database[int(i)]
		return ret


	def findCIDSubset(self, examples, meshTerms):
		"""
		Function findCIDSubset will return new list of examples (ids) containing examples which are annotated with at least one MeSH term from \
		list 'meshTerms'.
		"""
		newdata = []
		self.solo_att = self.__findMeshAttribute(examples)
		ids = list()
		meshTerms = list(set(meshTerms))
		# we couldn't find any mesh attribute
		if self.solo_att == "Unknown":
			return newdata
		for e in examples:
			try:
				ends = eval(e[self.solo_att].value)
			except SyntaxError:
				continue
			for i in meshTerms:
				if i in ends:
					newdata.append(e['cid'].value)
					break
		return newdata
	
	def findFrequentTerms(self, data, minSizeInTerm, treeData=False, callback=None):
		"""
		Function findFrequentTerms iterates thru examples in data. For every example it finds appropriate annotation (MeSH term). Result of this \
		function is a dictionary where keys corespond to MeSH terms ids and a value is a number of examples annotated with particular MeSH term.
		"""
		# we build a dictionary 		meshID -> [description, noReference, [cids] ]
		self.statistics = dict()
		self.calculated = False
		self.solo_att = self.__findMeshAttribute(data)
		# post processing variables
		ret = dict()
		ids = []
		succesors = dict()		# for each term id -> list of succesors
		succesors["tops"] = []
		# if we can't identify mesh attribute we return empty data structures
		if self.solo_att == "Unknown":
			if treeData:
				return succesors, ret
			else:
				return ret
		# plain frequency
		t = 0.0
		n = len(data)
		for i in data:
			t = t + 1
			if callback and t%10 == 0:
				callback(int(t*100/n))
			try:
				endNodes = list(set(eval(i[self.solo_att].value))) # for every CID we look up for end nodes in mesh. for every end node we have to find its ID	
			except SyntaxError:
				#print "Error in parsing ",i[self.solo_att].value
				continue
			# we find ID of end nodes
			endIDs = []
			for k in endNodes:
				if(self.toID.has_key(k)):	# this should be always true, but anyway ...
					endIDs.extend(self.toID[k])
				else:
					print "Current ontology does not contain MeSH term ", k, "." 
			# we find id of all parents
			#allIDs = self.__findParents(endIDs)
			allIDs = endIDs
			for k in allIDs:	# for every meshID we update statistics dictionary
				if(not self.statistics.has_key(k)):	# first time meshID
					self.statistics[k] = 0
				self.statistics[k] += 1	# counter increased
		# post processing
		for i in self.statistics.iterkeys():
			if(self.statistics[i] >= minSizeInTerm ): 
				ret[i] = self.statistics[i]
				ids.append(i)
		# we can also return tree data
		if treeData: #we return also compulsory data for tree printing 
			return self.__treeData(ids), ret 
		else:
			return ret

	def __treeData(self,ids):
		succesors = dict()
		succesors["tops"]= []
		# we build a list of succesors. for each node we know which are its succesors in mesh ontology
		for i in ids:
			succesors[i] = []
			for j in ids:
				if(i != j and self.__isPrecedesor(i,j)):
					succesors[i].append(j)
		# for each node from list above we remove its indirect succesors
		# only  i -1-> j   remain
		for i in succesors.iterkeys():
			succs = succesors[i]
			second_level_succs = []
			for k in succs:	 
				second_level_succs.extend(succesors[k])
			for m in second_level_succs:
				if succesors[i].count(m)>0:
					succesors[i].remove(m)
		# we make a list of top nodes
		tops = list(ids)
		for i in ids:
			for j in succesors[i]:
				tops.remove(j)
		# we pack tops table and succesors hash
		succesors["tops"] = tops
		return succesors  
		
	def findEnrichedTerms(self, reference, cluster, pThreshold=0.05, treeData=False, callback=None, fuzzy=False):
		"""
		Function findEnrichedTerms computes MeSH term enrichment based on a 'reference' and 'cluster' sets. It returns a dictionary where \
		keys are enriched (their p-value lower that 'pThreshold') MeSH terms. Key values are lists made of several items (MeSH term id, \
		MeSH term description, number of examples from the reference set, number of examples from the cluster set, p value, fold enrichment).
		"""
		self.clu_att = self.__findMeshAttribute(cluster)
		self.ref_att = self.__findMeshAttribute(reference)
		if((not self.calculated or self.reference != reference or self.cluster != cluster) and self.ref_att != "Unknown" and \
		self.clu_att != "Unknown"):	# Do have new data? Then we have to recalculate everything.
			self.reference = reference
			self.cluster = cluster			
			self.__calculateAll(callback, fuzzy)
		# declarations
		ret = dict()
		ids = []
		succesors = dict()		# for each term id -> list of succesors
		succesors["tops"] = []
		# if some attributes were unknown
		if (self.clu_att == "Unknown" or self.ref_att == "Unknown"):
			if treeData:
				return  succesors, ret
			else:
				return  ret
		for i in self.statistics.iterkeys():
			if(self.statistics[i][2] <= pThreshold ) :#or self.statistics[i][4] <= pThreshold ): # 
				ret[i] = self.statistics[i]
				ids.append(i)
		if treeData:
			return self.__treeData(ids),ret 
		else:
			return ret

	def printMeSH(self, data, selection=["term","r","c", "p"], func=None):
		"""
		Function printMeSH can be used to print result (dictionary) from the findEnrichedTerms and findFrequentTerms functions.
		"""
		# first we calculate additional info for printing MeSH ontology
		info = self.__treeData(data.keys())
		for i in info["tops"]:
			self.__pp(0,i,info,data, selection, funct = func)

	def __pp(self, offset, item, relations, data, selection, funct=None):
		mapping = {"term":0,"desc":1,"r":2,"c":3, "p":4, "fold":5, "func":6} 
		for i in range(0,offset):
			print " ",
		if type(data[item]) == list:
			pval = "%.4g" % data[item][2]
			fold = "%.4g" % data[item][3]
			print_data = [self.toName[item], self.toDesc[self.toName[item]], str(data[item][0]), str(data[item][1]), str(pval), str(fold)]
			for i in selection:
				if i != "term":
					print i + "=" + print_data[mapping[i]],
				else:
					print print_data[mapping[i]],
			if funct != None:
				print " ", funct(print_data[0]),
			#print self.toName[item], " r=" + str(data[item][1])  +" c="+ str(data[item][2])  ," p=" + str(pval) + " fold=" + str(fold)
			print ""
		else:
			print self.toName[item], " freq=" + str(data[item])
		for i in relations[item]:
			self.__pp(offset + 2, i, relations, data, selection, funct = funct) 

	def printHtmlMeSH(self, data, selection=["term","r","c", "p"], func=None):
		"""
		Function printHtmlMeSH if used to print results (dictinary) from the findFrequentTerms and findFrequentTerms functins in HTML format. \
		Together with the MeSH ontology it prints data like number of examples, p-values (enrichment).
		"""
		# first we calculate additional info for printing MeSH ontology
		info = self.__treeData(data.keys())
		w = {"term":"'95px'", "r":"'70px'","c":"'70px'","p":"'95px'"}
		print "<table>\n<tr>"
		for i in selection:
			print "<th width=" + w[i] +" align='left'>" + i  +"</th>"
		if func != None:
			func("header","")
		print "</tr>\n"
		for i in info["tops"]:
			self.__htmlpp(0,i,info,data, selection, funct = func)
		print "</table>"

	def __htmlpp(self, offset, item, relations, data, selection, funct = None):
		mapping = {"term":0,"desc":1,"r":2,"c":3, "p":4, "fold":5, "func":6} 
		print "<tr>"
		if type(data[item]) == list:
			pval = "%.4g" % data[item][2]
			fold = "%.4g" % data[item][3]
			print_data = [self.toName[item], self.toDesc[self.toName[item]], str(data[item][0]), str(data[item][1]), str(pval), str(fold)]
			for i in selection:
				print "<td>"
				if i == "term":
					for l in range(0,offset):
						print "&nbsp;",
				elif i == "p":
					print '%(#)2.3e' % {'#':float(print_data[mapping[i]])} + "</td>",
					continue
				print print_data[mapping[i]] + " &nbsp;</td>",

			if funct != None:
				print funct(print_data[0],item),

			#print self.toName[item], " r=" + str(data[item][1])  +" c="+ str(data[item][2])  ," p=" + str(pval) + " fold=" + str(fold)
			print "</tr>"
		else:
			print self.toName[item], " freq=" + str(data[item])

		for i in relations[item]:
			self.__htmlpp(offset + 2, i, relations, data, selection, funct = funct)

	def parsePubMed(self, filename, attributes=["pmid","title","abstract","mesh"], skipExamplesWithout=["mesh"]):
		"""
		Function parsePubMed can be used to parse (into Orange example table) PubMed search results (in XML).
		"""
		parser = make_parser()
		handler = pubMedHandler()
		parser.setContentHandler(handler)
		parser.parse(open(filename))
		atts = []
		for i in attributes:
			atts.append(orange.StringVariable(i))
		domain = orange.Domain(atts,0)
		data = orange.ExampleTable(domain)
		print data.domain.attributes
		mapping = {"pmid":0, "title":1, "abstract":2, "mesh":3, "affilation":4}
		for i in handler.articles:
			r=[]
			skip = False
			for f in attributes:
				if skipExamplesWithout.count(f) > 0:
					if (f == "mesh" and len(i[mapping[f]]) == 0) or str(i[mapping[f]]) == "":
						skip = True
				r.append(str(i[mapping[f]]))
			if not skip:
				data.append(r)
		return data	   

	def __findParents(self,endNodes):
		"""for each end node in endNodes function finds all nodes on the way up to the root"""		
		res = []
		for n in endNodes:
			tmp = n
			res.append(tmp)
			for i in range(n.count(".")):
				tmp = tmp.rstrip("1234567890").rstrip(".")
				if(tmp not in res):
					res.append(tmp)
		"""for i in res:
			nn = self.toID[self.toName[i]]
			for t in nn:
				if not t in res:
					res.append(t) """
		return res

	def __findMeshAttribute(self, data):
		"""
		Function tries to find attribute which contains list of MeSH terms.
		"""
		# we get a list of attributes
		dom = data.domain.attributes
		for i in dom:			  # for each attribute
			if i.varType == 6:
				for k in data:		   # for each domain
					att = str(i.name)
					try:										 # we try to use eval()
						r = eval(str(k[att].value))
						if type(r) == list:		 # attribute type should be list
							if self.dataLoaded:		 # if ontology is loaded we perform additional test
								for i in r:
									if self.toID.has_key(i): return att
							else:				   # otherwise we return list attribute
								return att
					except SyntaxError:
						continue
					except NameError:
						continue   
		print "Program was unable to determinate MeSH attribute."
		return "Unknown"

	def __isPrecedesor(self,a,b):
		""" function returns true if in Mesh ontology exists path from term id a to term id b """
		if b.count(a) > 0:
			return True
		return False

	def __calculateAll(self, callback, fuzzy):
		"""calculates all statistics"""
		# we build a dictionary 		meshID -> [description, noReference,noCluster, enrichment, deprivement, [cids] ]
		#print "ok"
		self.statistics = dict()
		if fuzzy:
			n = 0
			for i in self.reference:
				n = n + float(i['u'])
			cln = 0
			for i in self.cluster:
				cln = cln + float(i['u'])

		else:
			n = len(self.reference) 										# reference size
			cln = len(self.cluster)											# cluster size
		# frequency from reference list
		r = 0.0
		for i in self.reference:
			if callback and r%10 == 0:
				r += 1.0
				callback(int(100*r/(n+cln)))
			try:
				endNodes = list(set(eval(i[self.ref_att].value))) # for every CID we look up for end nodes in mesh. for every end node we have to find its ID	
			except SyntaxError:					 # where was a parse error
				print "Error in parsing ",i[self.ref_att].value
				if fuzzy:
					n = n - float(i["u"])
				else:
					n=n-1
				continue
			#we find ID of end nodes
			endIDs = []
			for k in endNodes:
				if(self.toID.has_key(k)):					# this should be always true, but anyway ...
					endIDs.extend(self.toID[k])
				else:
					print "Current ontology does not contain MeSH term ", k, "." 
			# endIDs may be empty > in this case we can skip this example
			if len(endIDs) == 0:
				if fuzzy:
					n = n - float(i["u"])
				else:
					n = n-1
				continue
			# we find id of all parents
			#allIDs = self.__findParents(endIDs) PATCH
			allIDs = endIDs
			for k in list(set(allIDs)):				# for every meshID we update statistics dictionary
				if(not self.statistics.has_key(k)):			# first time meshID
					self.statistics[k] = [ 0, 0, 0.0, 0.0 ]
				if fuzzy:
					self.statistics[k][0] += float(i["u"])
				else:
					self.statistics[k][0] += 1 # increased noReference
		# frequency from cluster list
		r=0.0
		for i in self.cluster:
			try:
				if callback and r%10 == 0:
					r += 1.0
					callback(int(100*r/(n+cln)))
				endNodes = list(set(eval(i[self.clu_att].value))) # for every CID we look up for end nodes in mesh. for every end node we have to find its ID	
			except SyntaxError:
				#print "Error in parsing ",i[self.clu_att].value
				if fuzzy:
					cln = cln - float(i["u"])
				else:
					cln = cln - 1
				continue
			# we find ID of end nodes
			endIDs = []
			for k in endNodes:
				if(self.toID.has_key(k)):				
					endIDs.extend(self.toID[k]) # for every endNode we add all corensponding meshIDs
			# endIDs may be empty > in this case we can skip this example
			if len(endIDs) == 0:
				if fuzzy:
					cln = cln - float(i["u"])
				else:
					cln = cln-1
				continue
			# we find id of all parents
			#allIDs = self.__findParents(endIDs)
			allIDs = endIDs
			for k in list(set(allIDs)): # for every meshID we update statistics dictionary
				if self.statistics.has_key(k):
					if fuzzy:
						self.statistics[k][1] += float(i["u"])
					else:
						self.statistics[k][1] += 1 # increased noCluster 
		self.ratio = float(cln)/float(n)
		# enrichment
		for i in self.statistics.iterkeys():
			self.statistics[i][2] = self.__calcEnrichment(int(n),int(cln),int(self.statistics[i][0]),int(self.statistics[i][1])) 	# p enrichment
			self.statistics[i][3] = float(self.statistics[i][1]) / float(self.statistics[i][0] ) / self.ratio   # fold enrichment
		self.calculated = True		

	def __calcEnrichment(self, n, c, t, tc):
		"""
		Function calculates hypergeometric distribution based on the input parameters.
		n - total number of items ie. size(cluster + reference)
		c - cluster size ie. size(cluster)
		t - number of all items in observed MeST term group
		tc - number of cluster chemicals in observed term group
		"""

		# FIXME: Popravi cudno racunanje enrichmenta v mejnih primerih.
		
		result=0
		for i in range(0,tc):
			result = result + exp(self.log_comb(t,i) + self.log_comb(n-t,c-i))
		result = result*1.0 / exp(self.log_comb(n,c))
		return (1.0-result)

	def log_comb(self,n, m): # it returns log(nCr(n,m))
		return self.lookup[n] - self.lookup[n-m] - self.lookup[m]

	def __loadOntologyFromDisk(self):
		""" 
		Function loads MeSH ontology and chemical annotation into internal data structures.
		"""
		self.toID = dict()  #   name -> [IDs] Be careful !!! One name can match many IDs!
		self.toName = dict() #   ID -> name
		self.toDesc = dict() # 	name -> description
		self.fromCID = dict() #   cid -> term id
		self.fromPMID = dict()  #   pmid -> term id
		
		__dataPath = orngServerFiles.localpath('MeSH')
		try:		
			# reading graph structure from file
			d = file(os.path.join(__dataPath,'mesh-ontology.dat'))
		except IOError:
			print os.path.join(__dataPath,'mesh-ontology.dat') + " does not exist!"
			return False
		try:	
			# reading cid annotation from file
			f = file(os.path.join(__dataPath,'cid-annotation.dat'))
		except IOError:
			print os.path.join(__dataPath,'cid-annotation.dat') + " does not exist!"
			#return False
		# loading ontology graph
		t=0
		for i in d:
			t += 1
			parts = i.split("\t") # delimiters are tabs
			if(len(parts) != 3):
				print "error reading ontology ", parts[0]
			parts[2] = parts[2].rstrip("\n\r")
			ids = parts[1].split(";")
			self.toID[parts[0]] = ids	# append additional ID
			self.toDesc[parts[0]] = parts[2]
			for r in ids:
				self.toName[r] = parts[0]
			# loading cid -> mesh
			for i in f:
				parts = i.split(";")		# delimiters are tabs
				if(len(parts) != 2):
					print "error reading ontology ", parts[0]
				parts[1] = parts[1].rstrip("\n\r")
				cid = int(parts[0])
				if self.fromCID.has_key(cid):
					self.fromCID[cid].append(parts[1])
				else:
					self.fromCID[cid] = [parts[1]]
			# loading pmid -> mesh, TODO
		#print "Current MeSH ontology contains ", t, " mesh terms."
		return True

class pubMedHandler(ContentHandler):
	def __init__(self):
		self.state = 0 	#  0 start state, 1 pmid, 2 title, 3 abstract, 4 mesh 
		self.articles = []
		self.pmid = "0"
		self.title = ""
		self.mesh = list()
		self.abstract = ""
		self.affiliation = ""

	def startElement(self, name, attributes):
		# print "parsam ", name
		if name == "PubmedArticle":
			self.pmid = ""
			self.abstract = ""
			self.title = ""
			self.mesh = []
		if name == "PMID":
			self.state = 1
		if name == "ArticleTitle":
			self.state = 2
		if name == "AbstractText":
			self.state = 3
		if name == "DescriptorName":
			self.state = 4
			self.mesh.append("")
		if name == "Affiliation":
			self.state = 5

	def characters(self, data):
		if self.state == 1:
			self.pmid += data
		if self.state == 2:
			self.title += data.encode("utf-8")
		if self.state == 3:
			self.abstract += data.encode("utf-8")
		if self.state == 4:
			self.mesh[-1] += data.encode("utf-8")
		if self.state == 5:
			self.affiliation += data.encode("utf-8")

	def endElement(self, name):
		#print "   koncujem ", name
		self.state = 0
		if name == "PubmedArticle":
			self.articles.append([self.pmid,self.title,self.abstract,self.mesh, self.affiliation])

class MappedMeSHParser(SGMLParser):
	def reset(self):
		self.pieces = []
		self.terms = []
		self.foundMeSH = False
		self.nextIsTerm = False
		self.endTags = ['Display','Write to the Help Desk', 'Previous Indexing:', 'Entry Terms:','Pharmacologic Action:' ]
		SGMLParser.reset(self)
	
	def unknown_starttag(self, tag, attrs):
		strattrs = "".join([' %s="%s"' % (key, value) for key, value in attrs])
		if self.foundMeSH and tag=='a':
			self.nextIsTerm = True
	
	def handle_data(self, text):
		text = text.strip()
		if text == '':
			return
		if text == 'Heading Mapped to:':
			self.foundMeSH = True
		if self.endTags.count(text)>0:
			self.foundMeSH = False
		elif self.nextIsTerm:
			self.terms.append(text)
			self.nextIsTerm = False

class PubChemMeSHParser(SGMLParser):
	def reset(self):
		self.next = 0
		self.nextLink = ''
		self.directTerms = []
		self.indirectTerms = []
		self.foundMeSH = False
		self.foundIndirectMeSH = 0
		SGMLParser.reset(self)
		# strategy as follows
		# Beetween strings "Drug and Chemical Info" and ("Pharmalogical Action" or "PubMed via MeSH" or "PubMed MeSH Keyword Summary") find hyperlinks. Based on title attribute we can distingue direct MeSH terms beetween mapped terms.

	def unknown_starttag(self, tag, attrs): 
		#if self.foundMeSH:
		#print 'tag ', tag, attrs
		if tag=='a' and len(attrs) > 2 and attrs[0][0] == 'name' and attrs[0][1] == 'gomesh': #print attrs
			self.nextLink = attrs[1][1]

	def handle_data(self, text):
		text = text.strip()
		if text == '':
			return
		#print 'data ', text
		if self.foundMeSH:
			#print '*'
			self.directTerms.append(text)
		
		if self.foundIndirectMeSH == 2:
			#print "indirect term", text, self.nextLink
			self.indirectTerms.append((text,self.nextLink))
			self.foundIndirectMeSH = 0
		
		if self.foundIndirectMeSH == 1:
			self.foundIndirectMeSH = 2
				
		if text == 'Drug and Chemical Information:':
			self.foundIndirectMeSH = 1


		if text == 'Pharmacological Action' or text == 'Medication Information':
			self.foundIndirectMeSH = 0
		if text == "Chemical Classification":
			self.foundMeSH = True
		elif (text == "Safety and Toxicology" or text=="Literature" or text == "Classification" or text == "PubMed via MeSH" or text == "PubMed MeSH Keyword Summary"):
			self.foundMeSH = False
			if len(self.directTerms) > 0:
				self.directTerms.pop()

class SmilesParser(SGMLParser):
	def reset(self):
		self.nextSmile = False
		self.smiles = ""
		SGMLParser.reset(self)

	def unknown_starttag(self, tag, attrs): 
		#print tag, " ", attrs
		if tag=="item":
			for (i,j) in attrs:
				if i=="name" and j=="CanonicalSmile":
					self.nextSmile = True

	def handle_data(self, text):
		if self.nextSmile:
			self.smiles = text
			self.nextSmile = False

class CIDsParser(SGMLParser):
	def reset(self):
		self.cid = False
		self.cids = []
		SGMLParser.reset(self)

	def unknown_starttag(self, tag, attrs): 
		#print tag, " ", attrs
		if tag=="id":
			self.cid = True
			
	def unknown_endtag(self, tag): 
		#print tag,
		if tag=="id":
			self.cid = False

	def handle_data(self, text):
		#print text
		if self.cid and not text == '\n\t\t':
			#print int(text.strip())
			self.cids.append(int(text))

class FormulaParser(SGMLParser):
	def reset(self):
		self.nextFormula = False
		self.formula = ""
		SGMLParser.reset(self)

	def unknown_starttag(self, tag, attrs): 
		#print tag, " ", attrs
		if tag=="item":
			for (i,j) in attrs:
				if i=="name" and j=="MolecularFormula":
					self.nextFormula = True

	def handle_data(self, text):
		if self.nextFormula:
			self.formula = text
			self.nextFormula = False

class CIDSParser(SGMLParser):
	def reset(self):
		self.nextCID = False
		self.cids = []
		SGMLParser.reset(self)

	def unknown_starttag(self, tag, attrs): 
		#print tag, " ", attrs
		if tag=="item":
			for (i,j) in attrs:
				if i=="name" and j=="int":
					self.nextCID = True

	def handle_data(self, text):
		if self.nextCID:
			self.cids.append(text)
			self.nextCID = False

class pubChemAPI(object):
	"""
	Class pubChemAPI is used to simplift the access to the pubChem database.
	"""
	def __init__(self):
		self.data = ""
		self.identifier = ""
		self.database = ""

	def getSMILE(self, id, typ):
		"""
		Function getSMILE takes chemicals id and its type and returns a SMILES representation. 
		"""
		dbs = {"sid":"pcsubstance", "cid":"pccompound"}
		if not dbs.has_key(typ):
			return "Unknown identifier"
		# maybe we already have the data ...
		if id == self.identifier and dbs[typ] == self.database:
			raw = self.data
		else:
			url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?db=" + dbs[typ] + "&id=" + str(id)
			#print url2fetch
			d = urlopen(url2fetch)
			raw = d.read()
			self.data = raw
			self.identifier = id
			self.database = dbs[typ]
		pr = SmilesParser()
		pr.feed(raw)
		data = pr.smiles
		pr.close()
		return data

	def getMeSHterms(self, cid):	
		"""
		Functions getMeSHterms tries to find MeSH annotation for given the CID in the pubChem database.
		"""
		usock = urlopen("http://pubchem.ncbi.nlm.nih.gov/summary/summary.cgi?cid="+ str(cid)  +"&viewopt=PubChem&hcount=100&tree=t#MeSH")
		parser = PubChemMeSHParser()
		text = usock.read()
		#print text
		parser.feed(text)
		usock.close()
		parser.close()
		allTerms = parser.directTerms
		#print allTerms
		for (k,i) in parser.indirectTerms:
			#print k, i
			#continue
			usock = urlopen(i)
			parser = MappedMeSHParser()
			parser.feed(usock.read())
			allTerms.extend(parser.terms)	 
			usock.close()
			parser.close()
		# from allTerms make a set
		allTerms = list(set(allTerms))
		return allTerms

	def getMolecularFormula(self, id, typ):
		"""
		Functions getMolecularFormula tries to find molecular formula for the given the chemical id and type in the pubChem database.
		"""
		dbs = {"sid":"pcsubstance", "cid":"pccompound"}
		if not dbs.has_key(typ):
			return "Unknown identifier"
		# maybe we already have the data ...
		if id == self.identifier and dbs[typ] == self.database:
			raw = self.data
		else:
			url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?db=" + dbs[typ] + "&id=" + str(id)
			#print url2fetch
			d = urlopen(url2fetch)
			raw = d.read()
			self.data = raw
			self.identifier = id
			self.database = dbs[typ]
		pr = FormulaParser()
		pr.feed(raw)
		data = pr.formula
		pr.close()
		return data

	def getCIDfromSID(self, sid):
		"""
		Functions getCIDfromSID converts compound id to substance id.
		"""
		dbs = {"sid":"pcsubstance", "cid":"pccompound"}
		if not dbs.has_key("sid"):
			return "Unknown identifier"
		# maybe we already have the data ...
		if sid == self.identifier and dbs["sid"] == self.database:
			raw = self.data
		else:
			url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?db=" + dbs["sid"] + "&id=" + str(sid)
			#print url2fetch
			d = urlopen(url2fetch)
			raw = d.read()
			self.data = raw
			self.identifier = sid
			self.database = dbs["sid"]
		pr = CIDSParser()
		pr.feed(raw)
		data = pr.cids
		pr.close()
		return data

	def getPharmActionList(self, cid):
		"""
		Functions getPharmActionList finds pharmacological MeSH annotation for the given CID.
		<Item Name="PharmActionList" Type="List">
		""" 
		dbs = {"sid":"pcsubstance", "cid":"pccompound"}
		# maybe we already have the data ...
		if cid == self.identifier and dbs["cid"] == self.database:
			raw = self.data
		else:
			url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?db=" + dbs["cid"] + "&id=" + str(cid)
			d = urlopen(url2fetch)
			raw = d.read()
		#print url2fetch
		self.data = raw
		self.identifier = cid
		self.database = dbs["cid"]
		ndata = raw.replace('\t','').replace('\n','')
		xmldoc = minidom.parseString(ndata)
		items = xmldoc.getElementsByTagName('Item')
		data = []
		for i in items:
			if i.attributes['Name'].value == 'PharmActionList':
				terms = i.childNodes
				for t in terms:
					data.append(str(t.childNodes[0].data))
		return data

	def getCIDs(self, name, weight):
		"""
		Functions getCIDs tries to find correct chemical id based on given chemical name and weight.
		"""
		offset = 0.005
		url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pccompound&term="
		url2fetch += name.replace(" ","%20") + "%20" + str(weight*(1-offset)) + ":" + str(weight*(1+offset)) + "[mw]&retmax=10000"
		#print url2fetch
		d = urlopen(url2fetch)
		raw = d.read()
		e = CIDsParser()
		e.feed(raw)
		return e.cids

	def getCIDs(self,name):
		"""
		Functions getCIDs tries to find correct chemical id based on given chemical name.
		"""
		#print "k"
		url2fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pccompound&term="
		url2fetch += name.replace(" ","%20") + "&retmax=10000"
		#print url2fetch
		d = urlopen(url2fetch)
		raw = d.read()
		e = CIDsParser()
		e.feed(raw)
		return e.cids

	def __strip_ml_tags__(self,in_text):
		s_list = list(in_text)
		i,j = 0,0
		while i < len(s_list):
			if s_list[i] == '<':
				while s_list[i] != '>':
					s_list.pop(i)
				s_list.pop(i)
			else:
				i=i+1
		join_char=''
		return join_char.join(s_list)



from __future__ import absolute_import, division

from collections import defaultdict
import cPickle, os, shutil, sys, StringIO, tarfile, urllib2

from Orange.orng import orngEnviron, orngServerFiles

from . import obiData, obiGenomicsUpdate

# list of common organisms from http://www.ncbi.nlm.nih.gov/Taxonomy
def common_taxids():
    """Return taxonomy IDs for common organisms."""
    return ["3702",  # Arabidopsis thaliana
            "9913",  # Bos taurus
            "6239",  # Caenorhabditis elegans
            "3055",  # Chlamydomonas reinhardtii
            "7955",  # Danio rerio (zebrafish)
            "352472", # Dictyostelium discoideum
            "7227",  # Drosophila melanogaster
            "562",   # Escherichia coli
            "11103", # Hepatitis C virus
            "9606",  # Homo sapiens
            "10090", # Mus musculus
            "2104",  # Mycoplasma pneumoniae
            "4530",  # Oryza sativa
            "5833",  # Plasmodium falciparum
            "4754",  # Pneumocystis carinii
            "10116", # Rattus norvegicus
            "4932",  # Saccharomyces cerevisiae
            "4896",  # Schizosaccharomyces pombe
            "31033", # Takifugu rubripes
            "8355",  # Xenopus laevis
            "4577",  # Zea mays
             ] 

def essential_taxids():
    """Return taxonomy IDs for organisms that are included in (default) Orange Bioinformatics installation."""
    return ["352472", # Dictyostelium discoideum
            "7227",  # Drosophila melanogaster
            "9606",  # Homo sapiens
            "10090", # Mus musculus
            "4932",  # Saccharomyces cerevisiae
            ] 

default_database_path = os.path.join(orngEnviron.bufferDir, "bigfiles", "Taxonomy")

class MultipleSpeciesException(Exception):
    pass

class UnknownSpeciesIdentifier(Exception):
    pass

def pickled_cache(filename=None, dependencies=[], version=1, maxSize=30):
    """ Return a cache function decorator. 
    """
    def datetime_info(domain, filename):
        try:
            return orngServerFiles.info(domain, filename)["datetime"]
        except IOError:
            return orngServerFiles.ServerFiles().info(domain, filename)["datetime"]
            
    def cached(func):
        default_filename = os.path.join(orngEnviron.bufferDir, func.__module__ + "_" + func.__name__ + "_cache.pickle")
        def f(*args, **kwargs):
            currentVersion = tuple([datetime_info(domain, file) for domain, file in dependencies]) + (version,)
            try:
                cachedVersion, cache = cPickle.load(open(filename or default_filename, "rb"))
                if cachedVersion != currentVersion:
                    cache = {}
            except IOError, er:
                cacheVersion, cache = "no version", {}
            allArgs = args + tuple([(key, tuple(value) if type(value) in [set, list] else value)\
                                     for key, value in kwargs.items()])
            if allArgs in cache:
                return cache[allArgs]
            else:
                res = func(*args, **kwargs)
                if len(cache) > maxSize:
                    del cache[iter(cache).next()]
                cache[allArgs] = res
                cPickle.dump((currentVersion, cache), open(filename or default_filename, "wb"), protocol=cPickle.HIGHEST_PROTOCOL)
                return res
        return f

    return cached

def cached(func):
    """Cached one arg method
    """
    def f(self, arg):
        if arg not in self._cache:
            self._cache[arg] = func(self, arg)
        return self._cache[arg]
    f._cache = {}
    f.__name__ = "Cached " + func.__name__
    return f
    
class TextDB(object):
    entry_start_string = chr(255)
    entry_end_string = chr(254)+"\n"
    entry_separator_string = chr(253)
    
    @property
    def _text_lower(self):
        if id(self._text) == self._lower_text_id:
            return self._lower_text
        else:
            self._lower_text_id = id(self._text)
            self._lower_text = self._text.lower()
            return self._lower_text
        
    def __init__(self, file=None, **kwargs):
        self._text = ""
        self._lower_text_id = id(self._text) - 1
        self._cache = {}
        self.__dict__.update(kwargs)
        
        if file != None:
            self._text = open(file, "rb").read()

    def _find_all(self, string, start=0, text=None, unique=True):
        text = text if text != None else self._text_lower
        while True:
            index = text.find(string, start)
            if index != -1:
                yield index
                if unique:
                    start = text.find(self.entry_start_string, index + 1)
                else:
                    start = index + 1
            else:
                raise StopIteration

    def _get_entry_at(self, index, text=None):
        text = text if text != None else self._text
        start = text.rfind(self.entry_start_string, 0, index + 1)
        end = text.find(self.entry_end_string, index)
        return self._text[start+1:end]

    @cached
    def get_entry(self, id):
        try:
            index = self._find_all(self.entry_start_string + id + self.entry_separator_string).next()
        except StopIteration:
            raise KeyError, id
        return self._get_entry_at(index)
                
    def search(self, string):
        string = string.lower()
        res = []
        for idx in self._find_all(string):
            entry = self._get_entry_at(idx)
            id , rest = entry.split(self.entry_separator_string, 1)
            self._cache[id] = entry
            res.append(id)
        return res

    def insert(self, entry):
        self._text += self.entry_start_string + self.entry_separator_string.join(entry) + self.entry_end_string

    def __iter__(self):
        for idx in self._find_all(self.entry_start_string):
            entry = self._get_entry_at(idx)
            if entry:
                yield entry.split(self.entry_separator_string ,1)[0]

    def __getitem__(self, id):
        entry = self.get_entry(id)
        return entry.split(self.entry_separator_string)[1:]

    def __setitem__(self, id, entry):
        self.insert([id] + list(entry))

    def create(self, filename):
        f = open(filename, "wb")
        f.write(self._text)
        def write(entry):
            f.write(self.entry_start_string + self.entry_separator_string.join(entry) + self.entry_end_string)
        return write
    
class Taxonomy(object):
    __shared_state = {"_text":None, "_info":None}
    def __init__(self):
        self.__dict__ = self.__shared_state
        if not self._text:
            self.Load()
            
    def Load(self):
        try:
            self._text = TextDB(os.path.join(default_database_path, "ncbi_taxonomy.tar.gz", "ncbi_taxonomy.db"))
            self._info = TextDB(os.path.join(default_database_path, "ncbi_taxonomy.tar.gz", "ncbi_taxonomy_inf.db"))
            return
        except Exception, ex:
            pass
        try:
            from . import orngServerFiles as sf
            sf.download("Taxonomy", "ncbi_taxonomy.tar.gz")
            self._text = TextDB(os.path.join(default_database_path, "ncbi_taxonomy.tar.gz", "ncbi_taxonomy.db"))
            self._info = TextDB(os.path.join(default_database_path, "ncbi_taxonomy.tar.gz", "ncbi_taxonomy_inf.db"))
            return
        except Exception, ex:
            raise

    def get_entry(self, id):
        try:
            entry = self._text[id]
        except KeyError:
            raise UnknownSpeciesIdentifier
        return entry
                
    def search(self, string, onlySpecies=True):
        res = self._text.search(string)
        if onlySpecies:
            res = [r for r in res if "species" in self._text[r][1]]
        return res

    def __iter__(self):
        return iter(self._text)

    def __getitem__(self, id):
        entry = self.get_entry(id)
        return entry[2] ## item with index 2 is allways scientific name

    def other_names(self, id):
        entry = self.get_entry(id)
        info = self._info[id]
        names = entry[2:] ## index 2 and larger are names
        return list(zip(names, info))[1:] ## exclude scientific name

    def rank(self, id):
        entry = self.get_entry(id)
        return entry[1]

    def parent(self, id):
        entry = self.get_entry(id)
        return entry[0]

    def subnodes(self, id, levels=1):
        res = self._text.search(self._text.entry_separator_string + id + self._text.entry_separator_string)
        res = [r for r in res if self.get_entry(r)[0] == id]
        if levels > 1:
            for r in list(res):
                res.extend(self.subnodes(r, levels-1))
        return res

    def taxids(self):
        return list(self)
    
    @staticmethod
    def ParseTaxdumpFile(file=None, outputdir=None, callback=None):
        from cStringIO import StringIO
        if file == None:
            file = tarfile.open(None, "r:gz", StringIO(urllib2.urlopen("ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz").read()))
        if type(file) == str:
            file = tarfile.open(file)
        names = file.extractfile("names.dmp").readlines()
        nodes = file.extractfile("nodes.dmp").readlines()
        namesDict = defaultdict(list)
        for line in names:
            if not line.strip():
                continue
            line = line.rstrip("\t\n|").split("\t|\t")
            id, name, unique_name, name_class = line
            if unique_name:
                namesDict[id].append((unique_name , name_class))
            else:
                namesDict[id].append((name , name_class))

        nodesDict = {}
        for line in nodes:
            if not line.strip():
                continue
            line = line.split("\t|\t")[:3]
            id, parent, rank = line
            nodesDict[id] = (parent, rank)
        
        name_class_codes = defaultdict(iter(range(255)).next)
        name_class_codes["scientific name"] ## Force scientific name to be first
        if outputdir == None:
            outputdir = default_database_path
        text = TextDB().create(os.path.join(outputdir, "ncbi_taxonomy.db"))
        info = TextDB().create(os.path.join(outputdir, "ncbi_taxonomy_inf.db"))
        milestones = set(range(0, len(namesDict), max(len(namesDict)/100, 1)))
        for i, (id, names) in enumerate(namesDict.items()):
            parent, rank = nodesDict[id]
            ## id, parent and rank go first
            entry = [id, parent, rank]
            ## all names and name class codes pairs follow ordered so scientific name is first
            names = sorted(names, key=lambda (name, class_): name_class_codes[class_])
            entry.extend([name for name ,class_ in names])
            info_entry = [id] + [class_ for name, class_ in names]
            text(entry)
            info(info_entry)
            if callback and i in milestones:
                callback(i)
    
def name(taxid):
    """ Return the scientific name for organism with taxid.
    """
    return Taxonomy()[taxid]

def other_names(taxid):
    """ Return a list of (name, name_type) tuples but exclude the scientific name.
    """
    return  Taxonomy().other_names(taxid)

@pickled_cache(None, [("Taxonomy", "ncbi_taxonomy.tar.gz")], version=1)
def search(string, onlySpecies=True, exact=False):
    """ Search the NCBI taxonomy database for an organism
    Arguments::
            - *string*      Search string
            - *onlySpecies* Return only taxids of species (and subspecies)
            - *exact*       Return only taxids of organism that exactly match the string
    """
    ids = Taxonomy().search(string, onlySpecies)
    if exact:
        ids = [id for id in ids if string in [name(id)] + [t[0] for t in other_names(id)]]
    return ids

def lineage(taxid):
    """ Return a list of taxids ordered from the topmost node (root) to taxid.
    """
    tax = Taxonomy()
    result = [taxid]
    while True:
        parent = tax.parent(result[-1])
        result.append(parent)
        if tax[parent] == "root" or parent=="1":
            break
    result.reverse()
    return result
    
def to_taxid(code, mapTo=None):
    """ See if the code is a valid code in any database and return a set of its taxids.
    """
    from . import obiKEGG, obiGO
    results = set()
    for test in [obiKEGG.to_taxid, obiGO.to_taxid]:
        try:
            r = test(code)
            if type(r) == set:
                results.update(r)
            else:
                results.add(r)
        except Exception, ex:
            pass

    if mapTo:
        mapped = [[parent_id for parent_id in mapTo if parent_id in lineage(r)].pop() for r in results if any(parent_id in lineage(r) for parent_id in mapTo)]
        if not mapped:
            t = Taxonomy()
            subnodes = dict([(r, t.subnodes(r, 4)) for r in results])
            mapped = [id for id in mapTo if any(id in subnodes[r] for r in results)]
        results = mapped

    return results

def taxids():
    return Taxonomy().taxids()

from . import obiGenomicsUpdate

class Update(obiGenomicsUpdate.Update):
    def GetDownloadable(self):
        return [Update.UpdateTaxonomy]
    
    def IsUpdatable(self, func, args):
        from datetime import datetime
        from . import obiData
        if func == Update.UpdateTaxonomy:
##            stream = urllib2.urlopen("ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz")
##            date = datetime.strptime(stream.headers.get("Last-Modified"), "%a, %d %b %Y %H:%M:%S %Z")
            ftp = obiData.FtpWorker("ftp.ncbi.nih.gov")
            size, date = ftp.statFtp("pub/taxonomy/taxdump.tar.gz")
            return date > self.GetLastUpdateTime(func, args)

    def UpdateTaxonomy(self):
        Taxonomy.ParseTaxdumpFile(outputdir=self.local_database_path)
        import tarfile
        tFile = tarfile.open(os.path.join(self.local_database_path, "ncbi_taxonomy.tar.gz"), "w:gz")
        tFile.add(os.path.join(self.local_database_path, "ncbi_taxonomy.db"), "ncbi_taxonomy.db")
        tFile.add(os.path.join(self.local_database_path, "ncbi_taxonomy_inf.db"), "ncbi_taxonomy_inf.db")
        tFile.close()

if __name__ == "__main__":
    ids = search("Homo sapiens")
    print ids
    print other_names(ids[0])
    



"""
<name>Normalization</name>
<description>Gene Expression data normalization</description>
<prototype>1</prototype>
"""

from __future__ import absolute_import

import os, sys

import Orange
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from ... import obiDifscale

class OWNormalization(OWWidget):
    settingsList = []
    
    METHODS = ["Median", "Quantile"]
    def __init__(self, parent=None, signalManager=None, title="Differentiation Scale"):
        OWWidget.__init__(self, parent, signalManager, title, wantMainArea=False)
        
        self.inputs = [("Data 1", Orange.data.Table, self.set_data1), ("Data 2", Orange.data.Table, self.set_data2)]
        self.outputs = [("Normalized Data 1", Orange.data.Table), ("Normalized Data 2", Orange.data.Table)]
        
        self.selected_method = "Median"
        
        #####
        # GUI
        #####
        
        OWGUI.comboBox(self.controlArea, self, "selected_method",
                       box="Method",
                       tooltip="Normalization type",
                       items=self.METHODS,
                       sendSelectedValue=True,
                       callback=self.on_method_changed)
        
        self.data1 = None
        self.data2 = None
        
        self.resize(200, 50)
        
    def set_data1(self, data=None):
        self.data1 = data
    
    def set_data2(self, data=None):
        self.data2 = data
        
    def handleNewSignals(self):
        self.run_normalization()
    
    def on_method_changed(self):
        self.run_normalization()
        
    def run_normalization(self):
        if self.data1 is not None:
            norm1, norm2 = obiDifscale.normalize(self.data1, self.data2, type=self.selected_method.lower())
            self.send("Normalized Data 1", norm1)
            self.send("Normalized Data 2", norm2)
        else:
            self.send("Normalized Data 1", None)
            self.send("Normalized Data 2", None)
            
if __name__ == "__main__":
    app = QApplication([])
    w = OWNormalization()
    data = Orange.data.Table("GDS")
    w.show()
    app.exec_()
